var serial = {};
var newReceivedData = false;
let globalReceviedBuffer = new Uint8Array(64);  // In the connect function there is a monitor on incoming data. This is where it ends up, consumed by the requester when newReceivedData = true.

(function() {
  'use strict';
  
                              
  serial.getPorts = function() {
    return navigator.usb.getDevices().then(devices => {
      return devices.map(device => new serial.Port(device));
    });
  };

  serial.requestPort = function() {
    const filters = [
      { 'vendorId': 0x04D8, 'productId': 0xEEAD },
    ];
    return navigator.usb.requestDevice({ 'filters': filters }).then(
      device => new serial.Port(device)
    );
  }

  serial.Port = function(device) {
    this.device_ = device;
  };

  serial.Port.prototype.connect = function() {
    return this.device_.open()
        .then(() => {
          if (this.device_.configuration === null) {
            return this.device_.selectConfiguration(1);
          }
        })
        .then(() => this.device_.claimInterface(0));
  };

  serial.Port.prototype.disconnect = function() {
    return this.device_.close();
  };

  serial.Port.prototype.send = function(data) {
    return this.device_.transferOut(1, data);
    // The calling function implements error catching
  };

  serial.Port.prototype.read = async function() {
    let result = await this.device_.transferIn(1, 64);
    var response_crc; // what the device said the CRC should be
    var crc;          // what we calculated the CRC should be
    var j;

    if (!serial.Port) {  // check if the port is open and exists
      console.log("Port closed or does not exist!");
      //status("Port error, please unplug and try again");
      return;
    }
    // Note: On a device disconnect (stall), data[] will be null triggering the desirable catch block
    for(j = 0; j < 64; j++){
      globalReceviedBuffer[j] = result.data.getUint8(j);    // Copy the received packet into nice and simple uint8 array
    }
    if(window.packet_data_debug){
      console.log(globalReceviedBuffer);
    }

    crc = crc16(globalReceviedBuffer, 62);
    response_crc = (globalReceviedBuffer[62] << 8) | globalReceviedBuffer[63];
    if(crc != response_crc){
      //console.log("Device returned an invalid CRC");
      throw("Device returned invalid CRC");
      
    } else {

      if(globalReceviedBuffer[0] == 255){
        //console.log("Device replied with packet error (code 255)");
        throw("Device returned packet error [" + globalReceviedBuffer[1] + "," + globalReceviedBuffer[2] + "]");
      } else {
        if(window.packet_data_debug){
          console.log("Device returned valid packet");  // The function that triggered this reply (eg: time set) will take it from here..
        }
        newReceivedData = true; // Now the caller function knows there is data to process. Any of the above error cases are handled here not there.
      }
    }
    return;
  }
})();