// WebSocket and USB Compatible Interface for NIXlabs Products
const RX_WAIT_TIMEOUT_MS = 250;
const USB_RX_WAIT_TIMEOUT_MS = 50;
const WS_RX_WAIT_TIMEOUT_MS = 500;  
const BUSY_WAIT_TIMEOUT_MS = 100;
const MODULE_RX_WAIT_TIMEOUT_MS = 3000;
const AWS_CONNECT_TIMEOUT_500MS = 40; // 20sec
const WIFI_CONNECT_TIMEOUT_500MS = 30;
const maxPacketRetries = 2;
// VVV Only for Full Web App VV //
const MODULE_WIFICONNECT_TIMEOUT_250MS = 60;  // in ms*250
const FIRMWARE_HEADER_SIZE = 0x90;
const MODULE_OFF = 2;
const MODULE_ON = 1;

const FW_MIN_SUPPORTED_MAJOR = 1;
const FW_MIN_SUPPORTED_MINOR = 3;
// RegEx pattern for any IP 0.0.0.0 to 255.255.255.255
const IPpattern = new RegExp(/^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/);

window.packet_data_debug = false;       // true= every packet and CRC check is shown in the console, only checked in transact_packet function
window.show_connection_checker = false; // true= even show the connection checker every 1s

window.FW_ChangeList = "";	
window.FW_LatestRev_major = 0;
window.FW_LatestRev_minor = 0;
window.FW_LatestRev_string = "";
window.NIXplus_LatestRev = 0;
NIXplus_LatestAppVer = 0;
window.NIXplus_ChangeList = ""
window.flashOp_progress = 0;  // Device flash operation percentage 0-1
// ^^ Only for Full Web App ^^ //

fwupg_percent = 0;
fwupg_stepsize = 0;
willUpgrade_NIXplus_app = false;
willUpgrade_NIXplus_fw = false;
willUpgrade_fw = false;
window.usingWS = false;
window.portBusy = false;  // set by sendPacket cleared by the calling function
window.ModelName = "?";
window.HasWiFiModule = false;
window.WiFiState = "";
window.WiFiOffDueToThrottling = false;
window.SerialNumber = 0;
window.FW_rev_major = 0;	// Major/minor are integers, FW_rew/HW_rev are strings
window.FW_rev_minor = 0;
window.BL_rev_major = 0;
window.BL_rev_minor = 0;
window.HW_rev_major = 0;
window.HW_rev_minor = 0;
window.NIXplus_FWver = 0;
window.NIXplus_AppVer = 0;
window.HW_rev = "";	
window.FW_rev = "";
var rtc_div;
var rtc_fdiv;
window.ALSnowValue = 0;

window.DeviceMode = ""; // "APPLICATION" or "BOOTLOADER"

(function() {

  var connectionChecker;

  var errCnt = 0;
  var errLast = 0;
  var meas5V = 0;
  var lastThrottle = 255;
  var throttle = 255;
  var ALSthresh = 0;
  var deviceTime = "";

  // Keep a local copy of the previous NIX+ credentials so we only re-write if they change
  var prev_DirSSID = "";
  var prev_DirPW = "";
  var prev_NetSSID = "";
  var prev_NetPW = "";
  var prev_Net_EN = false;
  var prev_Dir_EN = false;
  var prev_TZstr = "";
  //var prev_ADJstr = "";
  var WsRXdata = "";
  moduleDirectReadPending = false; 
  'use strict';
  
  // Mappings
  document.addEventListener('DOMContentLoaded', event => {
    let timeButton = document.querySelector("#submitTime");
    let antiButton = document.querySelector("#submitAnti");
    let ledButton = document.querySelector("#submitLED");
    let onoffButton = document.querySelector("#submitOnOff");
    let ALSthreshold = document.querySelector('#ALSthreshold');
    // VVV Only for Full Web App VV //
    window.connectButton = document.querySelector("#connect");
    window.connectWifiButton = document.querySelector("#connectWifi"); 
    window.wifiWSIP = document.querySelector("#WifiConnectIP");
    let port;
    // ^^ Only for Full Web App ^^ //

    const sleep = (milliseconds) => {
      return new Promise(resolve => setTimeout(resolve, milliseconds))
    }
        
    function setCheckbox(checkbox, value){
      if ( document.getElementById(checkbox).checked != value ) {
        document.getElementById(checkbox).click();
      }
    }

    // WebSocket Code for NIX+ models
     // WiFi Connect event
     window.connectWifiButton.addEventListener('click', function() {

      if( window.usingWS ){ // Websocket already open, so close it
        try{
          disconnect_tasks();
          ws.close();
        } catch (e){
          console.log("Attempted to close WebSocket, but object doesn't exist")
          // in case WS hasnt really been opened
        }

      } else {  // Websocket not currently open, so open it

        // If we are not HTTP then the button should say "WiFi Connection..." and not actually try to open a websocket
        if (location.protocol !== 'http:') {
          location.replace(`http:${location.href.substring(location.protocol.length)}`);
          return;
        }

        if (!("WebSocket" in window)) {
          console.log("Websocket not supported by browser, cannot continue");
          status("Browser not WebSocket compatible, use newer browser.");
          return;
        }

        if(window.wifiWSIP.value === "0.0.0.0"){
          ShowWiFiHelp();
          status("Error: The IP address is not valid");
          return;          
        }

        // Open WebSocket to NIX+ server module. It is also closed when the browser session ends.
        ws = new WebSocket('ws://' + window.wifiWSIP.value + "/websocket");
        ws.binaryType = "arraybuffer";
      }

      ws.onopen = function() {
        window.usingWS = true; 
        window.connectWifiButton.textContent = 'Disconnect WiFi';
        window.connectButton.disabled = true;
        window.wifiWSIP.disabled = true;
        window.portBusy = false;
        console.log("WebSocket Opened");
        connect_tasks();
        device_readAll();
      };
  
      ws.onclose = function() {
        disconnect_tasks();
        console.log("WebSocket Closed");
      };
  
      ws.onmessage = function(event) {

        if(event.data instanceof ArrayBuffer){
          var data = event.data;
          var dv = new DataView(data);
  
          var i;
          for(i=0; i<64; i++){
            globalReceviedBuffer[i] = dv.getUint8(i);
          }
          newReceivedData = true;
          if(window.packet_data_debug){
            console.log(globalReceviedBuffer);    // 64byte device packet
          }
        } else {
          if(window.packet_data_debug){
            console.log(event.data);          // information string
          }

          if(moduleDirectReadPending){      // WriteModule is awaiting a response from the module via websockets
            WsRXdata = event.data;          // shall be collected in the writeModule function
            moduleDirectReadPending = false;  // this is the flag for writeModule
          }
        }
      }
  
      ws.onerror = function(e) {
        disconnect_tasks();
        //console.log("Websocket error: ", e);
        status("Problem with the remote NIX+ device, please re-connect");
      }
      
      return;
    });

    // VV Only for full Web App VV //
    window.calibrate = async function(serial1, serial2, HWver, BLver, ClockCal, TempCal, RefCal) {  // Calibration command, factory setup use only (special FW)
      
      if(serial1 == null || serial2 == null || HWver == null || BLver == null || ClockCal == null || TempCal == null || RefCal == null){
        console.log("The correct usage is: calibrate(serial1, serial2, HWver, BLver, ClockCal, TempCal, RefCal)");
        return;
      }
      
      let toSendBuffer = new Uint8Array(64);
    
      toSendBuffer[0x00] = 18;

      toSendBuffer[0x30] = serial1;
      toSendBuffer[0x31] = serial2;
      toSendBuffer[0x32] = HWver;
      toSendBuffer[0x33] = BLver;
      toSendBuffer[0x34] = ClockCal;
      toSendBuffer[0x35] = TempCal;
      toSendBuffer[0x36] = RefCal;
      toSendBuffer[0x37] = 1;
     
      // Send and receive the packet
      console.log("Writing calibration values..");

      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      switch(globalReceviedBuffer[0x30]){
        case 0:
          console.log("Calibration not processed (not running factory test firmware)");
          status("Error writing calibration"); 
          break;
        case 1:
          console.log("Calibration rejected, the NVM space is already written");
          status("Error writing calibration"); 
          break;
        case 2:
          console.log("Calibration write error");
          status("Error writing calibration"); 
          break;
        case 3:
          console.log("Calibration verify error");
          status("Error writing calibration"); 
          break;
        case 4:
          console.log("Calibration set OK");
          status("Calibration set OK"); 
          break;
        default:
          console.log("An unknown calibration error occurred");
          status("Error writing calibration"); 
          break;
      }
      
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }

    window.clockOut = async function(mode=2) {  // Control the clock output pulse on RTCC/Set Pin. Set button will be disabled!
      // mode: 0=No change, 1=32.768kHz uncal output, 2=1Hz cal output, 3=Stop output

      let toSendBuffer = new Uint8Array(64);
    
      toSendBuffer[0x00] = 18;
      toSendBuffer[0x05] = 0;     // Temperature compensation 0=enabled, 1=disabled
      toSendBuffer[0x28] = 0;     // Warmup counter override. 0=no change, >0=apply the value less 1
      toSendBuffer[0x02] = mode;
     
      // Send and receive the packet
      console.log("Setting clock output mode " + mode);

      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(globalReceviedBuffer[0x02] == 1){
          console.log("OK");
      } else {
          console.log("Error, response was " + globalReceviedBuffer[0x02]);
      }
      
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }

	  window.getLatestFirmwareInfo = function() {	// Get the latest firmware details for this product 
    
      if (location.protocol !== 'https:') {
        console.log("Skipped latest firmware read as protocol is not HTTPS");

        if(location.protocol === 'file:'){  // Due to CORS firmware cannot be updated via local file, so repurpose this message space to warn about local file limitations
          document.getElementById("firmwareStatus").innerHTML = "NOTE: The app is running as a local file and will not be able to access some options under the gear icon.";  
        }
        
        return;
      }

      return new Promise(function (resolve, reject) {
        var xhr = new XMLHttpRequest();
        var NowDate = new Date(); // prevents caching
        var fwInfoAddress = 'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/firmwareInfo.json?v=';

        if(fwInfoAddress.includes("DEV")){
          console.warn("WARNING: You are using DEV_firmwareInfo.json !");
        }

        xhr.open('GET',	fwInfoAddress + NowDate.getTime(), true);
        xhr.onload = function () {
            if (xhr.status === 200) {
              
              var objFwInfo = JSON.parse(this.responseText);	
              var changeStr = "";

              switch (window.ModelName){
                case "DOT6":
                  window.FW_LatestRev_major = ((objFwInfo.DOT6_version & 0xC0) >> 6) +1;
                  window.FW_LatestRev_minor = (objFwInfo.DOT6_version & 0x3F);
                  window.FW_LatestRev_string = window.FW_LatestRev_major + "." + window.FW_LatestRev_minor;
                  
                  for(var versionInfo in objFwInfo.DOT6_changes){

                    // if device major rev is less than latest major rev, OR they're the same but the device minor rev is less than latest minor rev...
                    // If versions are identical then shall all changes ever
                    if(window.FW_rev_major < versionInfo.substring(1,2) || (window.FW_rev_major == versionInfo.substring(1,2) && window.FW_rev_minor < versionInfo.substring(3,4)) || (window.FW_rev_major == FW_LatestRev_major && window.FW_rev_minor == FW_LatestRev_minor) ){
                      if(changeStr !== ""){
                        changeStr += "<br>" // Add newline before next version's changes, except for the 1st line
                      }
                      changeStr += "<b><u>Changes in " + versionInfo.replace("_",".") + "</b></u><br>" + objFwInfo.DOT6_changes[versionInfo];
                    }
                  }
                  window.FW_ChangeList = changeStr;
                  break;
                  
                case "NIX6":
                  window.FW_LatestRev_major = ((objFwInfo.NIX6_version & 0xC0) >> 6) +1;
                  window.FW_LatestRev_minor = (objFwInfo.NIX6_version & 0x3F);
                  window.FW_LatestRev_string = window.FW_LatestRev_major + "." + window.FW_LatestRev_minor;
                  
                  for(var versionInfo in objFwInfo.NIX6_changes){

                    // if device major rev is less than latest major rev, OR they're the same but the device minor rev is less than latest minor rev...
                    // If versions are identical then shall all changes ever
                    if(window.FW_rev_major < versionInfo.substring(1,2) || (window.FW_rev_major == versionInfo.substring(1,2) && window.FW_rev_minor < versionInfo.substring(3,4)) || (window.FW_rev_major == FW_LatestRev_major && window.FW_rev_minor == FW_LatestRev_minor) ){
                      if(changeStr !== ""){
                        changeStr += "<br>" // Add newline before next version's changes, except for the 1st line
                      }
                      changeStr += "<b><u>Changes in " + versionInfo.replace("_",".") + "</b></u><br>" + objFwInfo.NIX6_changes[versionInfo];
                    }
                  }
                  window.FW_ChangeList = changeStr;
                  break;
      
                case "NIX4":
                  window.FW_LatestRev_major = ((objFwInfo.NIX4_version & 0xC0) >> 6) +1;
                  window.FW_LatestRev_minor = (objFwInfo.NIX4_version & 0x3F);
                  window.FW_LatestRev_string = window.FW_LatestRev_major + "." + window.FW_LatestRev_minor;
                  
                  for(var versionInfo in objFwInfo.NIX4_changes){

                    // if device major rev is less than latest major rev, OR they're the same but the device minor rev is less than latest minor rev...
                    // If versions are identical then shall all changes ever
                    if(window.FW_rev_major < versionInfo.substring(1,2) || (window.FW_rev_major == versionInfo.substring(1,2) && window.FW_rev_minor < versionInfo.substring(3,4)) || (window.FW_rev_major == FW_LatestRev_major && window.FW_rev_minor == FW_LatestRev_minor) ){
                      if(changeStr !== ""){
                        changeStr += "<br>" // Add newline before next version's changes, except for the 1st line
                      }
                      changeStr += "<b><u>Changes in " + versionInfo.replace("_",".") + "</b></u><br>" + objFwInfo.NIX4_changes[versionInfo];
                    }
                  }
                  window.FW_ChangeList = changeStr;
                  break;
                  
                case "VF4":
                  window.FW_LatestRev_major = ((objFwInfo.VF4_version & 0xC0) >> 6) +1;
                  window.FW_LatestRev_minor = (objFwInfo.VF4_version & 0x3F);
                  window.FW_LatestRev_string = window.FW_LatestRev_major + "." + window.FW_LatestRev_minor;
                  
                  for(var versionInfo in objFwInfo.VF4_changes){

                    // if device major rev is less than latest major rev, OR they're the same but the device minor rev is less than latest minor rev...
                    // If versions are identical then shall all changes ever
                    if(window.FW_rev_major < versionInfo.substring(1,2) || (window.FW_rev_major == versionInfo.substring(1,2) && window.FW_rev_minor < versionInfo.substring(3,4)) || (window.FW_rev_major == FW_LatestRev_major && window.FW_rev_minor == FW_LatestRev_minor) ){
                      if(changeStr !== ""){
                        changeStr += "<br>" // Add newline before next version's changes, except for the 1st line
                      }
                      changeStr += "<b><u>Changes in " + versionInfo.replace("_",".") + "</b></u><br>" + objFwInfo.VF4_changes[versionInfo];
                    }
                  }
                  window.FW_ChangeList = changeStr;
                  break;	

                case "FL4":
                  window.FW_LatestRev_major = ((objFwInfo.FL4_version & 0xC0) >> 6) +1;
                  window.FW_LatestRev_minor = (objFwInfo.FL4_version & 0x3F);
                  window.FW_LatestRev_string = window.FW_LatestRev_major + "." + window.FW_LatestRev_minor;
                  
                  for(var versionInfo in objFwInfo.FL4_changes){

                    // if device major rev is less than latest major rev, OR they're the same but the device minor rev is less than latest minor rev...
                    // If versions are identical then shall all changes ever
                    if(window.FW_rev_major < versionInfo.substring(1,2) || (window.FW_rev_major == versionInfo.substring(1,2) && window.FW_rev_minor < versionInfo.substring(3,4)) || (window.FW_rev_major == FW_LatestRev_major && window.FW_rev_minor == FW_LatestRev_minor) ){
                      if(changeStr !== ""){
                        changeStr += "<br>" // Add newline before next version's changes, except for the 1st line
                      }
                      changeStr += "<b><u>Changes in " + versionInfo.replace("_",".") + "</b></u><br>" + objFwInfo.FL4_changes[versionInfo];
                    }
                  }
                  window.FW_ChangeList = changeStr;
                  break;	
                
                default:
                  window.FW_LatestRev_major = 0;
                  window.FW_LatestRev_minor = 0;
                  window.FW_LatestRev_string = "Error reading model name";
                  window.FW_ChangeList = "Error reading model name";
                  console.log("Unknown model type during firmware info check");
                  sendHumio('url,'     + window.location.href           + '\n' +
                            'serial,'  + window.SerialNumber.toString() + '\n' +
                            'model,'  + window.ModelName.toString()     + '\n' +
                            'message,' + '"Webapp unknown model error type during firmware info check"' + '\n');
                  break;
              }
              // Get the NIX+ version, doesn't matter if its a NIX+ model or not
              window.NIXplus_LatestRev  = objFwInfo.NIXplus_version;
              NIXplus_LatestAppVer      = objFwInfo.NIXplus_spiffs_version;
              window.NIXplus_ChangeList = objFwInfo.NIXplus_changes;

              if(window.DeviceMode === "APPLICATION") {
                if( window.FW_rev_major < window.FW_LatestRev_major || (window.FW_rev_major == window.FW_LatestRev_major && window.FW_rev_minor < window.FW_LatestRev_minor) ){
        
                  if( window.FW_rev_major < FW_MIN_SUPPORTED_MAJOR || (window.FW_rev_major == FW_MIN_SUPPORTED_MAJOR && window.FW_rev_minor < FW_MIN_SUPPORTED_MINOR) ){
                    // This webapp version is not compatible with the installed FW so it must be upgraded
                    //alert("In order to use this WebApp, a device firmware update is required - it will take less than 30 seconds.");
                    document.getElementById("firmwareStatus").innerHTML = 'There is a firmware update available! <a href="#" id="lnk_openFW" onclick="openFWdialog();">Review changes and install.</a>';
                    document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now"; 
                    willUpgrade_fw = true;
                    // The firmware dialog box will be opened in device_readAll()
            
                  } else {
                    // For non-mandatory updates, just text is shown
                    document.getElementById("firmwareStatus").innerHTML = 'There is a firmware update available! <a href="#" id="lnk_openFW" onclick="openFWdialog();">Review changes and install.</a>';
                    document.getElementById("FWupdate_message").innerHTML = '';
                    document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
                    willUpgrade_fw = true;
                  }
                } else if (!willUpgrade_NIXplus_app && !willUpgrade_NIXplus_fw){ // this message space also used in module update code, don't say its all up to date if the module has populated this text
                  document.getElementById("firmwareStatus").innerHTML = "Your device has the latest firmware, enjoy!";  
                  document.getElementById("FWupdate_message").innerHTML = "You already have the latest version, but if you're experiencing problems you can try a re-install.";
                  document.getElementById("btn_FirmwareUpdate").innerHTML = "Re-Install Now";
                  willUpgrade_fw = false;
                }
              } else {
                willUpgrade_fw = true;
                willUpgrade_NIXplus_app = false;
                willUpgrade_NIXplus_fw = false;
              }

              if(window.HasWiFiModule == true){
                // The time estimate could be up to 1.5 minutes for updating the firmware, NIX+ flash and NIX+ app
                document.getElementById("FWupdate_bottomMsg").innerHTML = "You will be prompted to re-connected to your device once the firmware upgrade starts. The whole process will take less than 90 seconds.<font color='red'> Don't close or refresh your browser.</font>"
              } else {
                // Without a NIX+ module fitted it will only take 30 seconds
                document.getElementById("FWupdate_bottomMsg").innerHTML = "You will be prompted to re-connected to your device once the firmware upgrade starts. The whole process will take less than 30 seconds.<font color='red'> Don't close or refresh your browser.</font>"
              }
              resolve(xhr.response);

            } else {
              reject(xhr.status);
            }
        };
        xhr.send();
    });
  }
  // ^^ Only for full web app ^^ //

    async function device_readAll(){  // Read in all the devices info and current settings, set controls to the current settings
      
      await getDeviceInfo()
        .then(getLatestFirmwareInfo);
      
      if(window.DeviceMode === "APPLICATION") {
        console.log("Device connected successfully in application mode");
        var pcd = new Date();
        await getDeviceData_TimeDate()
        .then(getDeviceData_LEDsAndBrightness)
        .then(getDeviceData_AntipoisoningOptions)
        .then(getDeviceData_OnOffTimes)
        .then(getDeviceMeasurements)
        .then(getDeviceData_WifiModule)
        .then(getLatestFirmwareInfo);   // only for full webapp
        
        checkAutobrightSettings();    // Update the info messages for conflicting settings
        checkOnOffTimes();
        update_ALS_meas_pointer();    // update the ALS measurement pointer

        document.getElementById('ALSthresh').value = ALSthresh;

        if(window.HasWiFiModule){   // We have NIX+ model
          await getWiFiModuleStatus();
        }
         
        if( window.FW_rev_major < FW_MIN_SUPPORTED_MAJOR || (window.FW_rev_major == FW_MIN_SUPPORTED_MAJOR && window.FW_rev_minor < FW_MIN_SUPPORTED_MINOR) ){
          // This webapp version is not compatible with the installed FW so it must be upgraded
          await openFWdialog();
          document.getElementById("FWupdate_message").innerHTML = '<br><font color="red">In order to use this WebApp version, a device firmware update is required.</font>';
        }

        let pctime = pcd.getHours() + ":" + pcd.getMinutes() + ":" + pcd.getSeconds();

        // Send the settings file to Humio to help improve NIXlabs products. We don't share this info with anyone else - see our Privacy Policy
        var contentStr = await CreateSettingsString();
        sendHumio('url,'       + window.location.href           + '\n' +
                  'userAgent,"'+ navigator.userAgent            + '"\n' +
                  'language,'  + navigator.language             + '\n' +
                  'cookiesEnabled,' + navigator.cookieEnabled   + '\n' +
                  'resolution,'     + window.screen.width + 'x' + window.screen.height    + '\n' +
                  'timeDelta,"PC='  + pctime + ' Device=' + deviceTime + '"\n' +
                  'message,'  + '"Device connected successfully in application mode"' + '\n' + contentStr);

      } else if (window.DeviceMode === "BOOTLOADER"){
                 
       console.log("Device connected successfully in bootloader mode");
       if( !(document.getElementById("FWUpdateDialog").offsetHeight > 0 && document.getElementById("FWUpdateDialog").offsetWidth > 0) ) { // equivalent to jQuery not visible check
        await openFWdialog();
        document.getElementById("FWupdate_message").innerHTML = '<br><font color="red">Your device is in Bootloader Mode. Update your firmware here or unplug to exit Bootloader Mode without updating.</font>';
       }
       sendHumio('url,'    + window.location.href           + '\n' +
                'serial,' + window.SerialNumber.toString()  + '\n' +
                'model,'  + window.ModelName.toString()     + '\n' +
                'message,'+ '"Device connected successfully in bootloader mode"' + '\n');

      } else {
       console.error("Device returned unexpected value for operating mode");
       document.getElementById("FWupdate_message").innerHTML = '';

       sendHumio('url,'    + window.location.href            + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error - Device returned an unexpected value for operating mode"' + '\n');
     }
    }
    
    // Connect Function
    function connect() {

      // If we are not HTTPS or local file then the button should say "USB Connection..." and not actually try to open a usb device
      if ( !(location.protocol === 'https:' || location.protocol === 'file:') ) {
        location.replace(`https:${location.href.substring(location.protocol.length)}`);
        return;
      }

      port.connect().then(() => {
        window.connectButton.textContent = 'Disconnect USB';
        window.connectWifiButton.disabled = true;
        window.wifiWSIP.disabled = true;

        window.usingWS = false; // not using websocket, using USB
        window.portBusy = false;

        connect_tasks();
        device_readAll();
        
        connectionChecker = window.setInterval(async function(){
          // This function runs continuously until page refresh or disconnect is clicked. 
          // Don't care what operation we do, so long as its valid and we get some device response.
          // Could use diagnostics read but dont want to fill up the console (execution time would also be longer)

          // No need to run if diagnostics running (and they can interfere)
          if( (diagUpdater == undefined || diagUpdater == null) && !window.portBusy){
            let toSendBuffer = new Uint8Array(64);
            var prevPED = window.packet_data_debug;  // never show connection checker, it just clogs up the console
            window.packet_data_debug = window.show_connection_checker;
            
            toSendBuffer[0] = 0x10;  // The only non-flash command that is supported by both app and bootloader
            //console.log("- Connectivity Check -");
            await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
            newReceivedData = false;                // Dont care about the return data
            window.packet_data_debug = prevPED;     // Packet error debug mode back to previous state
          }

        }, 1000);

      }, error => {
        console.error(error);
      });
    }

    // USB Connect event
    window.connectButton.addEventListener('click', function() {
      if (port) {
        port.disconnect();
        disconnect_tasks();
        
      } else {

        // HTTPS required for USB, local file also works
        if ( !(location.protocol === 'https:' || location.protocol === 'file:') ) {
          location.replace(`https:${location.href.substring(location.protocol.length)}`);
        }

        serial.requestPort().then(selectedPort => {
          port = selectedPort;
          connect();
        }).catch(error => {
          console.error(error);
          disconnect_tasks();
        });
        if (port) {  // Connected OK

        }
      }
    });
    

    // Reconnect on refresh
    if ( (!!window.chrome) && (location.protocol === 'https:' || location.protocol === 'file:')) {
      serial.getPorts().then(ports => {
        if (ports.length == 0) {
          // Wasnt connected before
        } else {
          // Was connected, re-connect now
          port = ports[0];
          connect();
        }
      });
    }

    //  Submit Time
    timeButton.addEventListener('click', set_TimeDate);
    async function set_TimeDate() {

      document.getElementById("submitTime").disabled = true;  // disabled send button whilst we are processing

      console.log("Setting Date/Time");
      let toSendBuffer = new Uint8Array(64);

      toSendBuffer[0] = 1;  // Command to set time/date and general parameters. Other array vals are 0

      if( document.getElementById("nowTime").checked === true ) {
        var d = new Date();
        toSendBuffer[2] = d.getHours();
        toSendBuffer[3] = d.getMinutes();
        toSendBuffer[4] = d.getSeconds();
        toSendBuffer[5] = d.getFullYear().toString().substr(-2);
        toSendBuffer[6] = (d.getMonth()+1);
        toSendBuffer[7] = d.getDate();
        toSendBuffer[8] = d.getDay();
      } else {
        var time = document.getElementById("time").value;
        var date = document.getElementById("date").value;
        var day = new Date(date);
        //day = day.getDay();
        toSendBuffer[2] = time.substring(0,2);
        toSendBuffer[3] = time.substring(3,5);
        toSendBuffer[4] = time.substring(8,6);;
        toSendBuffer[5] = date.substring(2,4);
        toSendBuffer[6] = date.substring(5,7);
        toSendBuffer[7] = date.substring(8,10);
        toSendBuffer[8] = day.getDay();
      }

      if (document.getElementById("12hour").checked === true) {
        toSendBuffer[9] |= (1<<0);
      }
      if (document.getElementById("leadingZero").checked === true) {
        toSendBuffer[9] |= (1<<1);
      }
      if (document.getElementById("buttonBeep").checked === true) {
        toSendBuffer[9] |= (1<<2);
      }
      toSendBuffer[9] |= document.getElementById("displayMode").value << 3;  // changed in fv ver 1.1, only 2 bits wide now
      if (document.getElementById("showDateEN").checked === true) {
        toSendBuffer[9] |= (1<<5);
      }
      if (document.getElementById("fahrenheit").checked === true) {
        toSendBuffer[9] |= (1<<6);
      }
      if (document.getElementById("beepVol_hi").checked === true) {
        toSendBuffer[9] |= (1<<7);
      }

      toSendBuffer[10] |= document.getElementById("colonMode").value; 
      toSendBuffer[10] |= (0<<2);   // This is the bit for flickerEffect. Its supported but crappy so hidden. 1=on, 0=off
      if (document.getElementById("dateFormat_mmddyy").checked === true) {
        toSendBuffer[10] |= (1<<3);
      }
      if (document.getElementById("showTempEN").checked === true) {
        toSendBuffer[10] |= (1<<6);
      }
      if(window.ModelName === "VF4" || window.ModelName === "FL4"){  // not an option for non-7SEG models
        if (document.getElementById("tail_options_69").checked === true) {
          toSendBuffer[10] |= (1<<4);
        }
        if (document.getElementById("tail_options_7").checked) {
          toSendBuffer[10] |= (1<<5);
        }
      }

      // Valid threshold value range is 0.180V to 2.100V, which is already divided by 10 to fit in a byte over USB
      if(document.getElementById('ALSthresh').value >= 18 && document.getElementById('ALSthresh').value <= 210  ) {
        ALSthresh = document.getElementById('ALSthresh').value;
        toSendBuffer[0x0B] = ALSthresh;       // byte expects the desired threshold in mv/10. So 0.26V --> 26 and 1.5V --> 150.
        // The firmware will not commit the value if its not changed from the existing setting, so no harm in sending all the time
        // If the submitted value is out of range nothing will be set
      } else {
        toSendBuffer[0x0B] = 0; // Do nothing case
      }
      
      var threshMode;
      if( document.getElementById("nightThreshMode").value == 1) {
        // 1= Use time
        threshMode = 1;
      } else {
        // 0= Use ALS value
        threshMode = 0;
      }

      // Note that night start/end time format is different to the on-off timer format!
      var time = document.getElementById("nightStart").value;  // Night start time, MSB=1= use time instead of ALS for night determination
      if(time === ""){
        // Value is not set (ie: "--:--:--")
        toSendBuffer[0x0C] = 0; // 0 means use ALS for night detection
      } else {
        toSendBuffer[0x0C] = (threshMode<<7) | time.substring(0,2) << 2 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15. Both MSBs must be set to enable time-based
      }

      time = document.getElementById("nightEnd").value;  // Night end time, MSB=1= use time instead of ALS for night determination
      if(time === ""){
        // Value is not set (ie: "--:--:--")
        toSendBuffer[0x0D] = 0; // 0 means use ALS for night detection
      } else {
        toSendBuffer[0x0D] = (threshMode<<7) | time.substring(0,2) << 2 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15. Both MSBs must be set to enable time-based
      }

      if (document.getElementById("enable_tubeAutoBright").checked === true) {
        toSendBuffer[0x0E] |= 1 << 5;	
      }
      if (document.getElementById("enable_ledAutoBright").checked === true) {
        toSendBuffer[0x0E] |= 1 << 4;	
      }
      if (document.getElementById("displayFlip").checked === true) {
        toSendBuffer[0x0E] |= 1 << 6;	
      }

      // Send and receive the packet
      
      // Send the settings file to Humio to help improve NIXlabs products. We don't share this info with anyone else - see our Privacy Policy
      var contentStr = await CreateSettingsString();
      sendHumio('url,'    + window.location.href           + '\n' +
                'message,'+ '"Submitted the date/time settings"' + '\n' + contentStr);

      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat===1){
        console.log("Time/Date and options were set OK");
        status("Settings saved"); // All error messages handled by transact packet function

      } else if (stat===-1) {
        console.log("Device rejected one or more of the settings. Return codes =", globalReceviedBuffer[2], globalReceviedBuffer[3]);

        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error - Device rejected one or more of the settings during Set Time/Date. Code: ' + globalReceviedBuffer[2] + ',' + globalReceviedBuffer[3] + '"\n');
      }

      newReceivedData = false;  // data consumed, transact packet is possible again
      document.getElementById("submitTime").disabled = false;  // re-enable send button
      return;
    }
    
    /*  
    //  Set ALS threshold immediately
    ALSthreshold.addEventListener('click', ALSthresholdSet);
    async function ALSthresholdSet() {
      
      document.getElementById("ALSthreshold").disabled = true;  // disable set button whilst processing

      document.getElementById('ALSthresh').value = 0;   // If ALS value was imported, this action overrides it

      let toSendBuffer = new Uint8Array(64);
      toSendBuffer[0x00] = 8;   // this is the command that sets the LED and brightness levels
      // All other values are 0, meaning LEDs/brightness won't be set, only the ALS threshold
      toSendBuffer[0x0C] = 1;   // This means ALS threshold will be set to the current ambient light level 
            
      console.log("Setting ALS Threshold");

      let stat = await transact_packet(toSendBuffer);    // Check the port, calculate CRC and send the packet

      if(globalReceviedBuffer[2] === 8 && stat===1){
        console.log("ALS threshold set to current ambient level");
        status("Threshold set");

      } else if (stat===-1) {
        console.log("Device rejected one or more of the settings. Return codes =", globalReceviedBuffer[2], globalReceviedBuffer[3]);
        status("Error, please try again");

        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error - Device rejected one or more of the settings during Set ALS threshold. Code: ' + globalReceviedBuffer[2] + ',' + globalReceviedBuffer[3] + '"\n');
      }

      newReceivedData = false;  // reset, ready for next transaction

      await getDeviceMeasurements();  // Read the new actual ALS threshold point

      sendHumio('url,'      + window.location.href                     + '\n' +
                'serial,'   + window.SerialNumber.toString()           + '\n' +
                'model,'  + window.ModelName.toString()                + '\n' +
                'message,'  + '"Submitted the ALS threshold settings"' + '\n' +
                'ALSthresh,'+ (parseInt(ALSthresh*100)).toString()     + '\n');   // ALS is converted from Volts to mV/10 as accepted by the USB protocol

      document.getElementById("ALSthreshold").disabled = false;   // re-enable set button
      return;
    }
    */

    //  Read the current ALS level and update the slider
    ALSthreshold.addEventListener('click', ALSthresholdRead);
    async function ALSthresholdRead() {
      
      document.getElementById("ALSthreshold").disabled = true;  // disable set button whilst processing

      await getDeviceMeasurements();  // Read the current ALS level
      update_ALS_meas_pointer();
      console.log("ALS = " + window.ALSnowValue + " mV");

      document.getElementById("ALSthreshold").disabled = false;   // re-enable set button
      return;
    }

    /* For continuous ALS value streaming, execute this in the console 
        window.setInterval(async function(){
          await getDeviceMeasurements();  // Read the current ALS level
          window.update_ALS_meas_pointer();
          console.log("ALS = " + window.ALSnowValue + " mV");
        }, 1000);
    */

    window.update_ALS_meas_pointer = function(){
      // ALSnowValue is in mV, convert to a 0-100% for the pointer indicator
      if(window.ALSnowValue < 180){
        document.getElementById('meas_light_pointer').style.left = '0%';    // Slider range is 0.18V to 2.10V, but ALS could be anything so must clip value

      } else if (window.ALSnowValue > 2100){
        document.getElementById('meas_light_pointer').style.left = '100%';  // Slider range is 0.18V to 2.10V, but ALS could be anything so must clip value

      } else {
        document.getElementById('meas_light_pointer').style.left = (((window.ALSnowValue-180)/(2100-180)) * 100) + '%';
      }

      return;
    }
    
    
	//  Submit Antipoisoning options
	// Can specify the digitXX to set one or more digit at max brightness, for stronger antipoisoning
	// digitXX bit field: [7-6: Unused][5-4: mode 0-2][3-0: value 0-9]
	// "value" is digit 0-9. "mode" 0= blanked, 1= static number set by "value" 2= counting number
  // Set higherHV to >0 to use increased votlage
	// To clear display back to time, call without parameters (they default to all 0)
    antiButton.addEventListener('click', function(){
      window.set_AntipoisonOptions();
    });
    window.set_AntipoisonOptions = async function(digit10hr=0, digit1hr=0, digit10min=0, digit1min=0, digit10sec=0, digit1sec=0, higherHV=0) {
      
      document.getElementById("submitAnti").disabled = true;   // disable send button whilst processing

      let toSendBuffer = new Uint8Array(64);

      if (document.getElementById("antiMode").checked === true) {
        toSendBuffer[2] |= 1 << 0;	
      }
      if(document.getElementById("antiTime").value < 4){
        toSendBuffer[2] |= (document.getElementById("antiTime").value) << 1;	// Option 0-3 are "normal" but option 4 is a separate special byte

      } else if (document.getElementById("antiMode").checked === true){
        // Else its the command to start antipoisoning right now, but dont set if the global control is off otherwise weird stuff happens
        toSendBuffer[3] = 1;	
      }

      toSendBuffer[2] |= (document.getElementById("antiPattern").value)   << 3;
      toSendBuffer[2] |= (document.getElementById("antiLEDDuring").value) << 5;
      toSendBuffer[2] |= (document.getElementById("antiLEDAfter").value)  << 6;
      toSendBuffer[4]  = (document.getElementById("antiNight").value)  << 2;

      toSendBuffer[0] = 6;        // Instruction 
 
      // Now the single-digit antipoisoning.  If they are not specified they will be zero which is the normal state
      toSendBuffer[0x10] = digit10hr;
      toSendBuffer[0x11] = digit1hr;
      toSendBuffer[0x12] = digit10min;
      toSendBuffer[0x13] = digit1min;
      toSendBuffer[0x14] = digit10sec;	// 4-tube models will ignore seconds
      toSendBuffer[0x15] = digit1sec;
      
      toSendBuffer[0x16] = higherHV;

      // Send and receive the packet
      console.log("Setting Antipoisoning preferences");

      // Send the settings file to Humio to help improve NIXlabs products. We don't share this info with anyone else - see our Privacy Policy
      var contentStr = await CreateSettingsString();
      sendHumio('url,'    + window.location.href           + '\n' +
                'message,'+ '"Submitted the antipoisoning settings"' + '\n' + contentStr);

      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(globalReceviedBuffer[2] === 1 && stat===1){
        console.log("Antipoisoning options were set OK");
        status("Settings saved"); // All error messages handled by transact packet function

      } else if(globalReceviedBuffer[2] === 3 && stat===1){
        console.log("Digit antipoisoning is now running");
        //status("Digit antipoisoning is now running");

      } else if (stat===-1) {
        console.log("Device rejected one or more of the settings. Return codes =", globalReceviedBuffer[2], globalReceviedBuffer[3]);

        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error - Device rejected one or more of the settings during Set Antipoisoning. Code: ' + globalReceviedBuffer[2] + ',' + globalReceviedBuffer[3] + '"\n');
      }

      newReceivedData = false;  // data consumed, transact packet is possible again
      document.getElementById("submitAnti").disabled = false;   // re-enable button
      return;
    }
    

    //  Submit LED
    ledButton.addEventListener('click', set_LEDsAndBrightness);
    async function set_LEDsAndBrightness() {

      document.getElementById("submitLED").disabled = true;   // disable send button whilst processing
      
      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 8;    // This is the LED set instruction

      toSendBuffer[2] = document.getElementById("sliderGreen").value;
      toSendBuffer[3] = document.getElementById("sliderRed").value;
      toSendBuffer[4] = document.getElementById("sliderBlue").value;
      toSendBuffer[5] = 3;  // 3= save custom colour but dont change LEDs to this colour right now
	  
      toSendBuffer[0x10] = (document.getElementById("LEDcycler").value & 0x07);	// LED Colour cycler speed bits ...
      if (document.getElementById("enable_LEDcycler").checked === true) {			// ... and enable bit
      toSendBuffer[0x10] |= 1 << 3;	
      } else {
      toSendBuffer[0x10] |= 0 << 3;	
      }
      
      if( document.getElementById("ledTimeMode").value == 256){
      toSendBuffer[0x10] |= 1 << 4  // Special case for LED disabled
      }
      if( document.getElementById("ledDateMode").value == 256){
      toSendBuffer[0x10] |= 1 << 5  // Special case for LED disabled
      }
      if( document.getElementById("ledTempMode").value == 256){
      toSendBuffer[0x10] |= 1 << 6  // Special case for LED disabled
      }
      // Check the brightness sliders, zero is a prohibited value!
      if( document.getElementById("brightnessTimeMode").value == 0){
        document.getElementById("brightnessTimeMode").value = 1;	
      }
      if( document.getElementById("brightnessDateMode").value == 0){
        document.getElementById("brightnessDateMode").value = 1;	
      }
      if( document.getElementById("brightnessTempMode").value == 0){
        document.getElementById("brightnessTempMode").value = 1;	
      }
      toSendBuffer[6] = (document.getElementById("brightnessTimeMode").value << 3) | document.getElementById("ledTimeMode").value;
      toSendBuffer[7] = (document.getElementById("brightnessDateMode").value << 3) | document.getElementById("ledDateMode").value;
      toSendBuffer[8] = (document.getElementById("brightnessTempMode").value << 3) | document.getElementById("ledTempMode").value;
      
      toSendBuffer[9] = 1;  // 1= save these LED screen colours
      
      toSendBuffer[10] = (document.getElementById("NightLEDdimming").value << 6) | (document.getElementById("TubeDayBrightness").value << 3) | (document.getElementById("TubeNightBrightness").value);
      toSendBuffer[11] = 1; // 1= Save these tube brightness and LED night dimming settings
      
      toSendBuffer[12] = 0; // Set to '1' to use the current light level as the new day/night threshold
    
      // Send and receive the packet
      console.log("Setting LED and brightness preferences");

      // Send the settings file to Humio to help improve NIXlabs products. We don't share this info with anyone else - see our Privacy Policy
      var contentStr = await CreateSettingsString();
      sendHumio('url,'    + window.location.href           + '\n' +
                'message,'+ '"Submitted the LED and brightness settings"' + '\n' + contentStr);
        
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      // Return code says which 'groups' of settings were set. like LED colour, ALS threshold, screen colours etc can all be changed independantly
      if(globalReceviedBuffer[2] === 0x17 && stat===1){
        console.log("LED colours and brightness was set OK");
        status("Settings saved"); // All error messages handled by transact packet function

      } else if (stat===-1) {
        console.log("Device rejected one or more of the settings. Return codes =", globalReceviedBuffer[2], globalReceviedBuffer[3]);

        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error - Device rejected one or more of the settings during Set LEDs. Code: ' + globalReceviedBuffer[2] + ',' + globalReceviedBuffer[3] + '"\n');
      }

      newReceivedData = false;  // data consumed, transact packet is possible again
      document.getElementById("submitLED").disabled = false;   // re-enable button
      return;
    }
    
    
    // Read on-off times and populate controls
    async function getDeviceData_OnOffTimes() {
      var hh_on;
      var mm_on;
      var hh_off;
      var mm_off;
      var badValCnt;

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 0x05;  // Command that requests device on-off times. Other vals are 0
     
      console.log("Reading device On-Off times");
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat !== 1){
        newReceivedData = false;
        return; // didn't work so data isnt valid
      } 
            
      // Process the returned information and set the input controls
      
      // USB sleep/wake
      if(globalReceviedBuffer[0x20] & 0x01){
        setCheckbox("usbWake", true);
      } else {
        setCheckbox("usbWake", false);
      }
      if(globalReceviedBuffer[0x20] & 0x02){
        setCheckbox("usbSleep", true);
      } else {
        setCheckbox("usbSleep", false);
      }

      if (globalReceviedBuffer[0x20] & 0x80){
        document.getElementById("resumeLast").value = 1;
      }  else {
        document.getElementById("resumeLast").value = 0;
      }

      // Sunday on-off times 1
      hh_on  = globalReceviedBuffer[2] >> 3;
      mm_on  = (globalReceviedBuffer[2] & 3)*15;
      hh_off = globalReceviedBuffer[3] >> 3;
      mm_off = (globalReceviedBuffer[3] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[2] != 0xFF){
        document.getElementById('sun1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sun1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[3] != 0xFF){
        document.getElementById('sun1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sun1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('sun1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('sun1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Sunday on-off times 2
      hh_on  = globalReceviedBuffer[4] >> 3;
      mm_on  = (globalReceviedBuffer[4] & 3)*15;
      hh_off = globalReceviedBuffer[5] >> 3;
      mm_off = (globalReceviedBuffer[5] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[4] != 0xFF){
        document.getElementById('sun2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sun2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[5] != 0xFF){
        document.getElementById('sun2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sun2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('sun2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('sun2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Monday on-off times 1
      hh_on  = globalReceviedBuffer[6] >> 3;
      mm_on  = (globalReceviedBuffer[6] & 3)*15;
      hh_off = globalReceviedBuffer[7] >> 3;
      mm_off = (globalReceviedBuffer[7] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[6] != 0xFF){
        document.getElementById('mon1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('mon1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[7] != 0xFF){
        document.getElementById('mon1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('mon1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('mon1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('mon1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Monday on-off times 2
      hh_on  = globalReceviedBuffer[8] >> 3;
      mm_on  = (globalReceviedBuffer[8] & 3)*15;
      hh_off = globalReceviedBuffer[9] >> 3;
      mm_off = (globalReceviedBuffer[9] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[8] != 0xFF){
        document.getElementById('mon2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('mon2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[9] != 0xFF){
        document.getElementById('mon2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('mon2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('mon2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('mon2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Tuesday on-off times 1
      hh_on  = globalReceviedBuffer[10] >> 3;
      mm_on  = (globalReceviedBuffer[10] & 3)*15;
      hh_off = globalReceviedBuffer[11] >> 3;
      mm_off = (globalReceviedBuffer[11] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[10] != 0xFF){
        document.getElementById('tues1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('tues1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[11] != 0xFF){
        document.getElementById('tues1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('tues1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('tues1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('tues1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Tuesday on-off times 2
      hh_on  = globalReceviedBuffer[12] >> 3;
      mm_on  = (globalReceviedBuffer[12] & 3)*15;
      hh_off = globalReceviedBuffer[13] >> 3;
      mm_off = (globalReceviedBuffer[13] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[12] != 0xFF){
        document.getElementById('tues2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('tues2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[13] != 0xFF){
        document.getElementById('tues2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('tues2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('tues2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('tues2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Wednesday on-off times 1
      hh_on  = globalReceviedBuffer[14] >> 3;
      mm_on  = (globalReceviedBuffer[14] & 3)*15;
      hh_off = globalReceviedBuffer[15] >> 3;
      mm_off = (globalReceviedBuffer[15] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[14] != 0xFF){
        document.getElementById('wed1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('wed1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[15] != 0xFF){
        document.getElementById('wed1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('wed1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('wed1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('wed1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Wednesday on-off times 2
      hh_on  = globalReceviedBuffer[16] >> 3;
      mm_on  = (globalReceviedBuffer[16] & 3)*15;
      hh_off = globalReceviedBuffer[17] >> 3;
      mm_off = (globalReceviedBuffer[17] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[16] != 0xFF){
        document.getElementById('wed2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('wed2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[17] != 0xFF){
        document.getElementById('wed2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('wed2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('wed2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('wed2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Thursday on-off times 1
      hh_on  = globalReceviedBuffer[18] >> 3;
      mm_on  = (globalReceviedBuffer[18] & 3)*15;
      hh_off = globalReceviedBuffer[19] >> 3;
      mm_off = (globalReceviedBuffer[19] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[18] != 0xFF){
        document.getElementById('thurs1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('thurs1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[19] != 0xFF){
        document.getElementById('thurs1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('thurs1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('thurs1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('thurs1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Thursday on-off times 2
      hh_on  = globalReceviedBuffer[20] >> 3;
      mm_on  = (globalReceviedBuffer[20] & 3)*15;
      hh_off = globalReceviedBuffer[21] >> 3;
      mm_off = (globalReceviedBuffer[21] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[20] != 0xFF){
        document.getElementById('thurs2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('thurs2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[21] != 0xFF){
        document.getElementById('thurs2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('thurs2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('thurs2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('thurs2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Friday on-off times 1
      hh_on  = globalReceviedBuffer[22] >> 3;
      mm_on  = (globalReceviedBuffer[22] & 3)*15;
      hh_off = globalReceviedBuffer[23] >> 3;
      mm_off = (globalReceviedBuffer[23] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[22] != 0xFF){
        document.getElementById('fri1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('fri1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[23] != 0xFF){
        document.getElementById('fri1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('fri1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('fri1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('fri1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Friday on-off times 2
      hh_on  = globalReceviedBuffer[24] >> 3;
      mm_on  = (globalReceviedBuffer[24] & 3)*15;
      hh_off = globalReceviedBuffer[25] >> 3;
      mm_off = (globalReceviedBuffer[25] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[24] != 0xFF){
        document.getElementById('fri2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('fri2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[25] != 0xFF){
        document.getElementById('fri2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('fri2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('fri2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('fri2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Saturday on-off times 1
      hh_on  = globalReceviedBuffer[26] >> 3;
      mm_on  = (globalReceviedBuffer[26] & 3)*15;
      hh_off = globalReceviedBuffer[27] >> 3;
      mm_off = (globalReceviedBuffer[27] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[26] != 0xFF){
        document.getElementById('sat1_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sat1_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[27] != 0xFF){
        document.getElementById('sat1_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sat1_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('sat1', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('sat1', 1); // At least one time enabled, Slider set to 'enabled'
      }
      
      // Saturday on-off times 2
      hh_on  = globalReceviedBuffer[28] >> 3;
      mm_on  = (globalReceviedBuffer[28] & 3)*15;
      hh_off = globalReceviedBuffer[29] >> 3;
      mm_off = (globalReceviedBuffer[29] & 3)*15;
      badValCnt = 0;
      
      if(hh_on <= 23 && mm_on <= 45 && globalReceviedBuffer[28] != 0xFF){
        document.getElementById('sat2_on').value = ("00" + (hh_on).toString()).slice(-2) + ":" + ("00" + (mm_on).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sat2_on').value = "";  // invalid or unset value so clear the input field
      }
      if(hh_off <= 23 && mm_off <= 45 && globalReceviedBuffer[29] != 0xFF){
        document.getElementById('sat2_off').value = ("00" + (hh_off).toString()).slice(-2) + ":" + ("00" + (mm_off).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        badValCnt = badValCnt + 1;
        document.getElementById('sat2_off').value = "";  // invalid or unset value so clear the input field
      }

      if(badValCnt === 2){
        setCheckbox('sat2', 0); // Both times were out-of-range, disable the group
      } else {
        setCheckbox('sat2', 1); // At least one time enabled, Slider set to 'enabled'
      }
      newReceivedData = false;  // Data consumed
      
      return;
    }
    
    
//  Submit On-Off times
    onoffButton.addEventListener('click', set_OnOffTimes);
    async function set_OnOffTimes() {
      
      document.getElementById("submitOnOff").disabled = true;   // disable send button whilst processing
      
      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 4;    // This is the on-off times set instruction. Other vals are 0

      // USB wake/sleep
      if (document.getElementById("usbSleep").checked === true) {
        toSendBuffer[0x20] |= (1<<1);
      }
      if (document.getElementById("usbWake").checked === true) {
        toSendBuffer[0x20] |= (1<<0);
      }

      if (document.getElementById("resumeLast").value == 1) {
        toSendBuffer[0x20] |= (1<<7);
      }      

      // Sunday On-Off times 1
      var time = document.getElementById("sun1_on").value;
      if (document.getElementById("sun1").checked === true) {
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[2] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[2] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("sun1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[3] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[3] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[2] = 0xFF;   // Code for "not used"
        toSendBuffer[3] = 0xFF;   
      }
    
      // Sunday On-Off times 
      if (document.getElementById("sun2").checked === true) {
        time = document.getElementById("sun2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[4] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[4] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("sun2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[5] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[5] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[4] = 0xFF;   // Code for "not used"
        toSendBuffer[5] = 0xFF;   
      }

      
      // Monday On-Off times 1
      if (document.getElementById("mon1").checked === true) {
        time = document.getElementById("mon1_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[6] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[6] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("mon1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[7] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[7] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[6] = 0xFF;   // Code for "not used"
        toSendBuffer[7] = 0xFF;   
      }
    
      // Monday On-Off times 2
      if (document.getElementById("mon2").checked === true) {
        time = document.getElementById("mon2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[8] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[8] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("mon2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[9] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[9] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[8] = 0xFF;   // Code for "not used"
        toSendBuffer[9] = 0xFF; 
      }
      
      // Tuesday On-Off times 1
      if (document.getElementById("tues1").checked === true) {
        time = document.getElementById("tues1_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[10] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[10] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("tues1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[11] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[11] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[10] = 0xFF;   // Code for "not used"
        toSendBuffer[11] = 0xFF;   
      }
    
      // Tuesday On-Off times 2
      if (document.getElementById("tues2").checked === true) {
        time = document.getElementById("tues2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[12] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[12] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("tues2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[13] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[13] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[12] = 0xFF;   // Code for "not used"
        toSendBuffer[13] = 0xFF; 
      }
      
      // Wednesday On-Off times 1
      if (document.getElementById("wed1").checked === true) {
        time = document.getElementById("wed1_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[14] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[14] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("wed1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[15] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[15] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[14] = 0xFF;   // Code for "not used"
        toSendBuffer[15] = 0xFF;   
      }
    
      // Wednesday On-Off times 2
      if (document.getElementById("wed2").checked === true) {
        time = document.getElementById("wed2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[16] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[16] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("wed2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[17] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[17] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[16] = 0xFF;   // Code for "not used"
        toSendBuffer[17] = 0xFF; 
      }
      
      // Thursday On-Off times 1
      if (document.getElementById("thurs1").checked === true) {
        time = document.getElementById("thurs1_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[18] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[18] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("thurs1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[19] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[19] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[18] = 0xFF;   // Code for "not used"
        toSendBuffer[19] = 0xFF;   
      }
    
      // Thursday On-Off times 2
      if (document.getElementById("thurs2").checked === true) {
        time = document.getElementById("thurs2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[20] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[20] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("thurs2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[21] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[21] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[20] = 0xFF;   // Code for "not used"
        toSendBuffer[21] = 0xFF; 
      }
      
      // Friday On-Off times 1
      if (document.getElementById("fri1").checked === true) {
        time = document.getElementById("fri1_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[22] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[22] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("fri1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[23] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[23] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[22] = 0xFF;   // Code for "not used"
        toSendBuffer[23] = 0xFF;   
      }
    
      // Friday On-Off times 2
      if (document.getElementById("fri2").checked === true) {
        time = document.getElementById("fri2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[24] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[24] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("fri2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[25] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[25] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[24] = 0xFF;   // Code for "not used"
        toSendBuffer[25] = 0xFF; 
      }
      
      // Saturday On-Off times 1
      if (document.getElementById("sat1").checked === true) {
        time = document.getElementById("sat1_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[26] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[26] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);  // Hour and minutes/15
        }
        
        time = document.getElementById("sat1_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[27] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[27] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[26] = 0xFF;   // Code for "not used"
        toSendBuffer[27] = 0xFF;   
      }
    
      // Saturday On-Off times 2
      if (document.getElementById("sat2").checked === true) {
        time = document.getElementById("sat2_on").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[28] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[28] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
        time = document.getElementById("sat2_off").value;
        if(time == ""){
          // Value is not set (ie: "--:--:--")
          toSendBuffer[29] = 0xFF; // Code to disable this time
        } else {
          toSendBuffer[29] = time.substring(0,2) << 3 | ~~(time.substring(3,5) / 15);
        }
        
      } else {
        toSendBuffer[28] = 0xFF;   // Code for "not used"
        toSendBuffer[29] = 0xFF; 
      }

      console.log("Setting on-off timer preferences");

      // Send the settings file to Humio to help improve NIXlabs products. We don't share this info with anyone else - see our Privacy Policy
      var contentStr = await CreateSettingsString();
      sendHumio('url,'    + window.location.href           + '\n' +
                'message,'+ '"Submitted the on/off timer settings"' + '\n' + contentStr);

      let stat = await transact_packet(toSendBuffer);    // Check the port, calculate CRC and send the packet

      // Return code says which 'groups' of settings were set. like LED colour, ALS threshold, screen colours etc can all be changed independantly
      if(globalReceviedBuffer[2] === 1 && stat===1){ 
        console.log("On-Off times were set OK");
        status("Settings saved");

      } else if (stat===-1) {
        console.log("Device rejected one or more of the settings. Return codes =", globalReceviedBuffer[2], globalReceviedBuffer[3]);
        status("Error, please try again");

        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error - Device rejected one or more of the settings during Set On/Off Timer. Code: ' + globalReceviedBuffer[2] + ',' + globalReceviedBuffer[3] + '"\n');
      }
      newReceivedData = false;

      document.getElementById("submitOnOff").disabled = false;   // re-enable button
      return;
    }
    

    
    window.getDeviceInfo = async function( dontUpdateModel=false) {
      
      let toSendBuffer = new Uint8Array(64);
      
      // assume that all other elements are set to 0
      toSendBuffer[0] = 0x10;  // this is the command that requests device info

      console.log("Reading device info");
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat !== 1){
        newReceivedData = false;

        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Function getDeviceInfo() failed due to data packet error ' + + globalReceviedBuffer[0x00] + ',' + globalReceviedBuffer[0x01] + ',' + globalReceviedBuffer[0x02] + '"\n');
        return; // didn't work so data isnt valid
      } 
            
      if(globalReceviedBuffer[0x30] != 0){
        window.DeviceMode = "BOOTLOADER";   // Firmware update mode
        
      } else {
        window.DeviceMode = "APPLICATION";  // Running in applicaiton mode (normal operation)
      }

      // This model icon may be updated later to show its a "+" model (with WiFi) so only modify the badge if it is not yet set
      if( dontUpdateModel === false ){
      // Use "Model" to also hide certain web app elements, eg: DOT6 has no colon option
      document.getElementById('colon_options').style.display = "block";			// Default case is to show these options
      document.getElementById('leadingzero_options').style.display = "block";	// 
      document.getElementById('vf4_tail_options').style.display = "none";	    // Default is to hide these
      document.getElementById('displayFlip_options').style.display = "none";

        switch(globalReceviedBuffer[0x02]) {
          case 1:
            document.getElementById("statusText").innerHTML = "DOT6 Connected";
            window.ModelName = "DOT6";
            document.getElementById('colon_options').style.display = "none";			// Hide colon options for DOT6
            document.getElementById('leadingzero_options').style.display = "none";	// Hide leading zero option for DOT6
            break;

          case 2:
            document.getElementById("statusText").innerHTML = "NIX4 Connected";
            window.ModelName = "NIX4";
            break;

          case 3:
            document.getElementById("statusText").innerHTML = "NIX6 Connected";
            window.ModelName = "NIX6";
            break;
        
          case 4:
            document.getElementById("statusText").innerHTML = "VF4 Connected";
            window.ModelName = "VF4";
            document.getElementById('vf4_tail_options').style.display = "block";		// Show the tails option which only applies for 7SEG type clocks
            break;  

          case 5:
            document.getElementById("statusText").innerHTML = "FL4 Connected";
            window.ModelName = "FL4";
            document.getElementById('vf4_tail_options').style.display = "block";		// Show the tails option which only applies for 7SEG type clocks
            document.getElementById('displayFlip_options').style.display = "block";
            break;    

          case 6:
            document.getElementById("statusText").innerHTML = "WD1 Connected";
            window.ModelName = "WD1";
            break;    

          default:
            document.getElementById("statusText").innerHTML = "Unsupported Model";
            window.ModelName = "Unknown";

            sendHumio('url,'    + window.location.href           + '\n' +
                      'serial,' + window.SerialNumber.toString() + '\n' +
                      'model,'  + window.ModelName.toString()    + '\n' +
                      'message,'+ '"Function getDeviceInfo() received unknown model ' + globalReceviedBuffer[0x02] + '"\n');
            break;
        }

        // Populate the antipoisoning at night select box based on model type
        document.getElementById('antiNight').options.length = 0;  // remove all

        if(globalReceviedBuffer[0x02] === 5){
          // FL4 doesnt increase brightness for 'antipoison' so remove Dimmed at Night option and rename "Normal (Full Brightness)" to just "Normal"
          document.getElementById('antiNight').add(new Option('Normal',                   '0'));
          document.getElementById('antiNight').add(new Option('Off at night',             '2'));

        } else { // All other models
          document.getElementById('antiNight').add(new Option('Normal (full brightness)', '0'));
          document.getElementById('antiNight').add(new Option('Dimmed at night',          '1'));
          document.getElementById('antiNight').add(new Option('Off at night',             '2'));
        }
      } else {
        // sendHumio('url,'    + window.location.href           + '\n' +
        //           'serial,' + window.SerialNumber.toString() + '\n' +
        //           'model,'  + window.ModelName.toString()    + '\n' +
        //           'message,'+ '"Function getDeviceInfo() was instructed not to update model ' + globalReceviedBuffer[0x02] + '"\n');
      }

      window.SerialNumber = globalReceviedBuffer[0x03] << 8 | globalReceviedBuffer[0x04]; // 16 bit serial number. YYWWs, YY=year WW=week s=sequence (0-9)
        
      // Hardware revision (including mods): [major 7-4][minor 3-0], where major=0= PCB "A-J" and modifications range from 0(none) to 15
      window.HW_rev_major = (globalReceviedBuffer[0x06] >> 4) + 65;		// These individual parts are used in the firmware upgrade compatibility check
      window.HW_rev_minor = (globalReceviedBuffer[0x06] & 0x0F);
      window.HW_rev = String.fromCharCode(window.HW_rev_major) + window.HW_rev_minor.toString();
      
      // Bootloader revision: [major 7-6][minor 5-0] with +1 added to major section. eg v1.15 = 15(dec), v3.5 = 133(dec)
      window.BL_rev_major = (((globalReceviedBuffer[0x0B] & 0xC0) >> 6) +1);			// These individual parts are used in the firmware upgrade compatibility check
      window.BL_rev_minor = (globalReceviedBuffer[0x0B] & 0x3F);
      
      // Firmware revision: [major 7-6][minor 5-0] with +1 added to major section. eg v1.15 = 15(dec), v3.5 = 133(dec)
      window.FW_rev_major = (((globalReceviedBuffer[0x07] & 0xC0) >> 6) +1);			// These individual parts are used in the firmware upgrade compatibility check
      window.FW_rev_minor = (globalReceviedBuffer[0x07] & 0x3F);
      window.FW_rev = (((globalReceviedBuffer[0x07] & 0xC0) >> 6) +1).toString() + "." + (globalReceviedBuffer[0x07] & 0x3F).toString();     
      
      if( window.FW_rev_major > 1 || (window.FW_rev_major == 1 && window.FW_rev_minor >= 6) ){   // if version 1.6 or greater
        // The following features are only available in firmware v1.6 or greater
        document.getElementById('option_tubeAutoBright').style.display = "block"; 
        document.getElementById('option_ledAutoBright').style.display = "block";
        document.getElementById('option_resumeLast').style.display = "block";
      } else {
        document.getElementById('option_tubeAutoBright').style.display = "none";
        document.getElementById('option_ledAutoBright').style.display = "none";
        document.getElementById('option_resumeLast').style.display = "none";
      }

      errCnt  = globalReceviedBuffer[0x09] << 8 | globalReceviedBuffer[0x0A];  // 16 bit total errors
      errLast = globalReceviedBuffer[0x2E];

      // Display blanking, throttling and weather disabled by on-off timer ([colhr][colsec][10hr][1hr] [10min][1min][10sec][1sec])
      throttle        = globalReceviedBuffer[0x2D];
      var DispModeStr = "Blanking= " + (("00" + globalReceviedBuffer[0x2C].toString(16)).slice(-2)).toUpperCase() + ", Throttling= " + throttle + "/9, ";
      
      if(globalReceviedBuffer[0x08] & 0x01){
        DispModeStr += "On-Off Timer: Active";
      } else {
        DispModeStr += "On-Off Timer: Inactive";
      }

      if(globalReceviedBuffer[0x2F] & 0x80){
        lastThrottle = globalReceviedBuffer[0x2F] & 0x7F;
        DispModeStr += "<br>Previous Throttling= " + lastThrottle + "/9";
      } // else, this FW version doesnt have this feature, or no data is available yet


      try {  // The fw_ fields are null until the firmware upgrade window has been loaded.
        document.getElementById("fw_ModelName").innerText = window.ModelName;
        document.getElementById("fw_SerialNumber").innerText =  window.SerialNumber.toString();
        document.getElementById("fw_FWrev").innerText   =  window.FW_rev;  // device installed rev
      } catch (err){
        // No problem. Just means diagnostics window not open
      }

      try {  // The diag_ fields are null until the diagnostics window has been loaded.
        document.getElementById("diag_SerialNumber").innerText = SerialNumber.toString();
        document.getElementById("diag_HWrev").innerText = String.fromCharCode((globalReceviedBuffer[0x06] >> 4) + 65) + (globalReceviedBuffer[0x06] & 0x0F); 
        document.getElementById("diag_BLrev").innerText = (((globalReceviedBuffer[0x0B] & 0xC0) >> 6) +1).toString() + "." + (globalReceviedBuffer[0x0B] & 0x3F).toString();    
        document.getElementById("diag_FWrev").innerText =  window.FW_rev;

        // CPU Unique ID (UDID) 160b
        var strUDID = ("00" + globalReceviedBuffer[0x18].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x19].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x1A].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x1B].toString(16)).slice(-2)   
                    + ("00" + globalReceviedBuffer[0x1C].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x1D].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x1E].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x1F].toString(16)).slice(-2)
                    + ("00" + globalReceviedBuffer[0x20].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x21].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x22].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x23].toString(16)).slice(-2) 
                    + ("00" + globalReceviedBuffer[0x24].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x25].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x26].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x27].toString(16)).slice(-2) 
                    + ("00" + globalReceviedBuffer[0x28].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x29].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x2A].toString(16)).slice(-2) + ("00" + globalReceviedBuffer[0x2B].toString(16)).slice(-2);
        
        document.getElementById("diag_UDID").innerText = strUDID.toUpperCase();
        
        switch(globalReceviedBuffer[0x08] & 0xF0){
          case 0x80:
            document.getElementById("diag_HVstat").innerText = "Off";
            break;
          case 0x40:
            document.getElementById("diag_HVstat").innerText = "Soft-Starting";
            break;  
          case 0x20:
            document.getElementById("diag_HVstat").innerText = "Striking";
            break;  
          case 0x10:
            document.getElementById("diag_HVstat").innerText = "On";
            break;  
          default:
            document.getElementById("diag_HVstat").innerText = "?";
            break; 
        }
        
        document.getElementById("diag_DispMode").innerHTML = DispModeStr;

        // Flash page erase count, E=Event log, S=Settings.  Note, Event log erase counter is cleared on error clear command.
        document.getElementById("diag_FlashCycles").innerText =  "Event Log: " + (globalReceviedBuffer[0x31] << 8 | globalReceviedBuffer[0x32]).toString() + ", Settings: " + (globalReceviedBuffer[0x33] << 8 | globalReceviedBuffer[0x34]).toString();
        document.getElementById("diag_ErrCount").innerText = errCnt.toString() + ", Last: " + errLast;  // Also show last error code recorded
      } catch (err){
        // No problem. Just means diagnostics window not open
      }
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }
    
    
    
    window.getDeviceMeasurements = async function() {
      
      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 0x11;  // Command to requests device measurements, other vals are 0
      
      console.log("Reading device measurements");
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat !== 1){
        newReceivedData = false;
        return; // didn't work so data isnt valid
      } 
       
      meas5V       = ((globalReceviedBuffer[0x0E] << 8 | globalReceviedBuffer[0x0F])/1000).toFixed(3);
      ALSthresh    = (globalReceviedBuffer[0x1F] << 8 | globalReceviedBuffer[0x20])/10;
      window.ALSnowValue  = globalReceviedBuffer[0x08] << 8 | globalReceviedBuffer[0x09];
      rtc_div      = globalReceviedBuffer[0x2D];  // Actual RTC major divider. For perfect clock it would be 0x3FFF, and each count is worth about 61.035ppm
      rtc_fdiv     = globalReceviedBuffer[0x2E];  // Actual RTC fine divider.  For perfect clock it would be 0, and each count is worth about 1.907ppm

      // If the 5V ripple measurement code is enabled then you can use the following, by default it is not enabled. When ON_TIME is <4000 the sampling rate is too short and the reading will be zero.
      //console.log("5V Ripple Vpp = " + (((globalReceviedBuffer[0x36] << 8 | globalReceviedBuffer[0x37])/1000) - ((globalReceviedBuffer[0x38] << 8 | globalReceviedBuffer[0x39])/1000)).toFixed(3) + "V");

      try { // The diag_ fields are null until the diagnostics window has been loaded.

        var digLen = 2;
        if(window.ModelName === "WDX" || window.ModelName === "FL4"){
          // These models do not have an HV converter so the fields are not used (read as 0)
          document.getElementById('diag_Row_HVmeas').style.display = "none";
          document.getElementById('diag_row_HVset').style.display = "none";
          document.getElementById('diag_row_HVduty').style.display = "none";
          document.getElementById('diag_row_GateSense').style.display = "none";
          digLen = 3; // brighter leds go up to 255 (otherwise max is 31)
        }

        document.getElementById("diag_HVmeas").innerText =  ((globalReceviedBuffer[0x02] << 8 | globalReceviedBuffer[0x03])/10).toFixed(1) + "V";           // Set HV
        document.getElementById("diag_HVset").innerText =  ((globalReceviedBuffer[0x04] << 8 | globalReceviedBuffer[0x05])/10).toString() + "V";            // Actual HV
        document.getElementById("diag_HVduty").innerText =  (globalReceviedBuffer[0x06] << 8 | globalReceviedBuffer[0x07]).toString() + "%";                // HV drive duty
        document.getElementById("diag_ALS").innerText =  (window.ALSnowValue/1000).toFixed(3) + "V (" + ((globalReceviedBuffer[0x29])?("Day)"):("Night)"));                          // ALS plus device's ALS_isDay flag
        document.getElementById("diag_NightThresh").innerText =  (ALSthresh/100).toString() + "V";     
        document.getElementById("diag_RGBLevels").innerHTML = (globalReceviedBuffer[0x31]).toString().padStart(digLen,"0") + "," + (globalReceviedBuffer[0x30]).toString().padStart(digLen,"0") + "," + (globalReceviedBuffer[0x32]).toString().padStart(digLen,"0") + ". DimS/C = " + (globalReceviedBuffer[0x33]).toString() + "/" + (globalReceviedBuffer[0x34]).toString() + ". Corr = " + (globalReceviedBuffer[0x2F] << 8 | globalReceviedBuffer[0x27]).toString();;

        if( window.FW_rev_major > 1 || (window.FW_rev_major == 1 && window.FW_rev_minor >= 6) ){   // if version 1.6 or greater
          // In v1.6 and newer, the display method is improved and the diagnostics info has changed slightly
          // Tube on-time, total period, colon on-time, extra_brightness
          document.getElementById("diag_DispControl").innerText =  "Ton= "  + (globalReceviedBuffer[0x21] << 8 | globalReceviedBuffer[0x22]).toString() 
                                + ",  Period= " + (globalReceviedBuffer[0x23] << 8 | globalReceviedBuffer[0x24]).toString()
                                + ",  Con= "    + (globalReceviedBuffer[0x25] << 8 | globalReceviedBuffer[0x26]).toString()
                                + ",  EB= "     + (globalReceviedBuffer[0x35]).toString();
        } else {
          document.getElementById('diag_row_RGBLevels').style.display = "none";     // FW <1.6 don't have the RGB/ALS Correction diagnostics info 
          // Tube on-time, tube off-time, colon on-time
          document.getElementById("diag_DispControl").innerText =  "Ton= "  + (globalReceviedBuffer[0x21] << 8 | globalReceviedBuffer[0x22]).toString() 
                                + ",  Toff= " + (globalReceviedBuffer[0x23] << 8 | globalReceviedBuffer[0x24]).toString()
                                + ",  Con= "  + (globalReceviedBuffer[0x25] << 8 | globalReceviedBuffer[0x26]).toString();
        }

        if(window.ModelName === "FL4"){
          // This model has display current exceeding the 16b capacity, so has to be scaled up x8
          document.getElementById("diag_DispCurrent").innerText =  ((globalReceviedBuffer[0x0A] << 8 | globalReceviedBuffer[0x0B])/125).toFixed(3) + "mA";   // Scaled display current
        } else {
          document.getElementById("diag_DispCurrent").innerText =  ((globalReceviedBuffer[0x0A] << 8 | globalReceviedBuffer[0x0B])/1000).toFixed(3) + "mA";   // HV display current
        }
        document.getElementById("diag_GateSense").innerText =  ((globalReceviedBuffer[0x0C] << 8 | globalReceviedBuffer[0x0D])/1000).toFixed(3) + "V";      // Gate Sense
        document.getElementById("diag_VUSB").innerText = meas5V.toString() + "V";                                                                           // 5V USB
        document.getElementById("diag_VCC").innerText  =  ((globalReceviedBuffer[0x10] << 8 | globalReceviedBuffer[0x11])/1000).toFixed(3) + "V";           // CPU VCC
        document.getElementById("diag_RefV").innerText =  "Ref: " + ( ( (globalReceviedBuffer[0x12] & 0x80) ? ((globalReceviedBuffer[0x12] << 8 | globalReceviedBuffer[0x13]) - 65536) : (globalReceviedBuffer[0x12] << 8 | globalReceviedBuffer[0x13]) )/1000).toString()
                + "V,  Temp: " + ( (globalReceviedBuffer[0x2A] & 0x80) ? ((globalReceviedBuffer[0x2A] - 256) * 0.05) : (globalReceviedBuffer[0x2A] * 0.05) ).toFixed(2) + "\u00B0C of " + ( (globalReceviedBuffer[0x2B] & 0x80) ? ((globalReceviedBuffer[0x2B] - 256) * 0.05) : (globalReceviedBuffer[0x2B] * 0.05) ).toFixed(2)
                + "\u00B0C,  Clock: " + (globalReceviedBuffer[0x2C]).toString();  // CPU programmed (calibrated) reference voltage, Temp cal (instantanous and final) and clock cal
        document.getElementById("diag_Temp").innerText =  ((globalReceviedBuffer[0x14] << 8 | globalReceviedBuffer[0x15])/1000).toString() + "\u00B0C";     // Temperature
        document.getElementById("diag_InstLoopTime").innerText =  (((globalReceviedBuffer[0x17] << 24) | (globalReceviedBuffer[0x18] << 16) | (globalReceviedBuffer[0x19] << 8) | (globalReceviedBuffer[0x1A]))/10).toFixed(1) + "\u00B5s";  // Instantaneous main loop time 
        document.getElementById("diag_MaxLoopTime").innerText =   (((globalReceviedBuffer[0x1B] << 24) | (globalReceviedBuffer[0x1C] << 16) | (globalReceviedBuffer[0x1D] << 8) | (globalReceviedBuffer[0x1E]))/10).toString() + "\u00B5s";   // 1min Maximum main loop time 
        document.getElementById("diag_BtnStatus").innerText =  ("00000000" + (globalReceviedBuffer[0x16]).toString(2)).slice(-8);                         // Push button states       
      } catch (err){
        // No problem. Just means diagnostics window not open
      }
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }    

    
    async function getDeviceData_TimeDate() {
      
      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 0x02;  // Command that requests device time/date, other vals are 0
      
      console.log("Reading device time/date");
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat !== 1){
        newReceivedData = false;
        return; // didn't work so data isnt valid
      } 
            
      // Process the returned information and set the input controls
      if(globalReceviedBuffer[0x09] & 0x01){  // 12-24hr time mode
        document.getElementById("12hour").click();
      } else {
        document.getElementById("24hour").click(); 
      }
  
      if(globalReceviedBuffer[0x09] & 0x02){  // ZeroPad on/off
        setCheckbox("leadingZero", true);
      } else {
        setCheckbox("leadingZero", false);
      }
  
      if(globalReceviedBuffer[0x09] & 0x04){  // Beep on/off
        setCheckbox("buttonBeep", true);
      } else {
        setCheckbox("buttonBeep", false);
      }

      deviceTime = globalReceviedBuffer[0x02] + ":" + globalReceviedBuffer[0x03] + ":" + globalReceviedBuffer[0x04];
      
      switch( ((globalReceviedBuffer[0x09] & 0x18) >> 3) ){  // Display mode time
        case 0:
        default:
          document.getElementById("displayMode").value = 0; // Time only
          break;
          
        case 1:
          document.getElementById("displayMode").value = 1; // every 30s show..
          break;
          
        case 2:
          document.getElementById("displayMode").value = 2; // every minute show..
          break;
          
        case 3:
          document.getElementById("displayMode").value = 3; // every 5min show..
          break;
      }
      if( document.getElementById("displayMode").value == 0){
        document.getElementById('dispModeOptions').style.display = "none";
      } else {
        document.getElementById('dispModeOptions').style.display = "";
      }

      if( globalReceviedBuffer[0x09] & 0x20 ){
        setCheckbox("showDateEN", true);     // auto show date
      } else {
        setCheckbox("showDateEN", false);    // no date
      }

      if( globalReceviedBuffer[0x0A] & 0x40 ){
        setCheckbox("showTempEN", true);     // auto show temp
      } else {
        setCheckbox("showTempEN", false);    // no temp
      }
  
      if(globalReceviedBuffer[0x09] & 0x40){  // Temperature C/F
        document.getElementById("fahrenheit").click();  // Selects C and deselects F
      } else {
        document.getElementById("celsius").click();  // Selects F and deselects C
      }
	  
	    if(globalReceviedBuffer[0x09] & 0x80){  // Beep volume low/high
        document.getElementById("beepVol_hi").click();  // Selects High
      } else {
        document.getElementById("beepVol_lo").click();  // Selects Low
      }
	  
  
      switch( (globalReceviedBuffer[0x0A] & 0x03) ){
        case 0:
        default:
          document.getElementById("colonMode").value = 0;
          break;
          
        case 1:
          document.getElementById("colonMode").value = 1;
          break;
        
        case 2:
          document.getElementById("colonMode").value = 2;
          break;
          
        case 3:
          document.getElementById("colonMode").value = 3;
          break;  
      }
	  
      /*	The below flicker effect is not very good, but all code is there..  Search for "flickerEffect" in index.html
      if(globalReceviedBuffer[0x0A] & 0x04){  // Flicker effect
          setCheckbox("flickerEffect", true);
        } else {
          setCheckbox("flickerEffect", false);
        }
      */
	  
	    if(globalReceviedBuffer[0x0A] & 0x08){  // Date format DDMMYY/MMDDYY
        document.getElementById("dateFormat_mmddyy").click();  // Selects MMDDYY
      } else {
        document.getElementById("dateFormat_ddmmyy").click();  // Selects DDMMYY
      }
	  
	    if(globalReceviedBuffer[0x0A] & 0x10){  // Tails on 6/9
        setCheckbox("tail_options_69", true);  // With tails
      } else {
        setCheckbox("tail_options_69", false);  // without
      }
	  
	    if(globalReceviedBuffer[0x0A] & 0x20){  // Tails on 7
        setCheckbox("tail_options_7", true);  // With tail
      } else {
        setCheckbox("tail_options_7", false);  // without
      }

      // Time-based night detection
      var temp_hh = (globalReceviedBuffer[0x0C] & 0x7C) >> 2;   // Night start time
      var temp_mm = (globalReceviedBuffer[0x0C] & 0x03)*15;
      var badval = false;
      if( temp_hh <= 23 && temp_mm <= 45 ){
        document.getElementById('nightStart').value = ("00" + (temp_hh).toString()).slice(-2) + ":" + ("00" + (temp_mm).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        document.getElementById('nightStart').value = "00:00";  // bad value
        badval = true;
      }  

      temp_hh = (globalReceviedBuffer[0x0D] & 0x7C) >> 2;       // Night end time
      temp_mm = (globalReceviedBuffer[0x0D] & 0x03)*15;
      if( temp_hh <= 23 && temp_mm <= 45 ){
        document.getElementById('nightEnd').value = ("00" + (temp_hh).toString()).slice(-2) + ":" + ("00" + (temp_mm).toString()).slice(-2);  // Accepts string "HH:MM"
      } else {
        document.getElementById('nightEnd').value = "00:00";  // bad value
        badval = true;
      } 

      if( globalReceviedBuffer[0x0C] & 0x80 && globalReceviedBuffer[0x0D] & 0x80 && !badval){
        // If both MSBs are set then we are using time-based night detection
        document.getElementById("nightThreshMode").value = 1;
        document.getElementById('Night_ALSbased').style.display = "none";
        document.getElementById('Night_TimeBased').style.display = "";
        document.getElementById('ALS_slider').style.display = "none";
      } else {
        // Using ALS
        document.getElementById("nightThreshMode").value = 0;
        document.getElementById('Night_ALSbased').style.display = "";
        document.getElementById('Night_TimeBased').style.display = "none";
        document.getElementById('ALS_slider').style.display = "";
      }

      if(globalReceviedBuffer[0x0E] & 0x10){
        setCheckbox("enable_ledAutoBright", true);
      } else {
        setCheckbox("enable_ledAutoBright", false);
      }

      if(globalReceviedBuffer[0x0E] & 0x20){
        setCheckbox("enable_tubeAutoBright", true);
      } else {
        setCheckbox("enable_tubeAutoBright", false);
      }

      if(globalReceviedBuffer[0x0E] & 0x40){
        setCheckbox("displayFlip", true);
      } else {
        setCheckbox("displayFlip", false);
      }

      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }
    
    
    async function getDeviceData_LEDsAndBrightness() {
      
      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 0x09;  // Command to request device LED and tube brightness settings. Other vals are 0
      
      console.log("Reading device LED and brightness settings");
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat !== 1){
        newReceivedData = false;
        return; // didn't work so data isnt valid
      } 
            
      document.getElementById("sliderGreen").value = globalReceviedBuffer[2];   // Custom LED colour
      document.getElementById("sliderRed").value = globalReceviedBuffer[3];
      document.getElementById("sliderBlue").value = globalReceviedBuffer[4];
  
      // LEDs disabled bits. There are three individual screen off bits and the 0x80 bit is all screens off due to button setting
      if(globalReceviedBuffer[0x10] & 0x10 || globalReceviedBuffer[0x10] & 0x80){	
        document.getElementById("ledTimeMode").value = 256; // Over-ride the colour selector to show the off state.  The brightness slider will stay unchanged though
      } else {
        document.getElementById("ledTimeMode").value = globalReceviedBuffer[6] & 0x07;			  // LEDs are not disabled so populate the colour selector
      }
      if(globalReceviedBuffer[0x10] & 0x20 || globalReceviedBuffer[0x10] & 0x80){	
        document.getElementById("ledDateMode").value = 256;  
      } else {
        document.getElementById("ledDateMode").value = globalReceviedBuffer[7] & 0x07;
      }
      if(globalReceviedBuffer[0x10] & 0x40 || globalReceviedBuffer[0x10] & 0x80){	
        document.getElementById("ledTempMode").value = 256; 
      } else {
        document.getElementById("ledTempMode").value = globalReceviedBuffer[8] & 0x07;
      }
      document.getElementById("brightnessTimeMode").value = (globalReceviedBuffer[6] & 0xF8) >> 3;    // LED Brightnesses
      document.getElementById("brightnessDateMode").value = (globalReceviedBuffer[7] & 0xF8) >> 3;
      document.getElementById("brightnessTempMode").value = (globalReceviedBuffer[8] & 0xF8) >> 3;
       
      document.getElementById("TubeNightBrightness").value = globalReceviedBuffer[10] & 0x07;
      document.getElementById("TubeDayBrightness").value = (globalReceviedBuffer[10] & 0x38) >> 3;
      document.getElementById("NightLEDdimming").value = (globalReceviedBuffer[10] & 0xC0) >> 6;
  
      document.getElementById("LEDcycler").value = globalReceviedBuffer[0x10] & 0x07;	// LED cycler speed bits

      if(globalReceviedBuffer[0x10] & 0x08){	// LED cycler enable bit. Update checkbox and enable/disbale screen colour selection
        setCheckbox('enable_LEDcycler', 1);
        document.getElementById("ledTimeMode").disabled = true;
        document.getElementById("ledTimeMode").classList.add("disabled");
        document.getElementById("ledDateMode").disabled = true;
        document.getElementById("ledDateMode").classList.add("disabled");
        document.getElementById("ledTempMode").disabled = true;
        document.getElementById("ledTempMode").classList.add("disabled");
      } else {
        setCheckbox('enable_LEDcycler', 0); 
        document.getElementById("ledTimeMode").disabled = false;
        document.getElementById("ledTimeMode").classList.remove("disabled");
        document.getElementById("ledDateMode").disabled = false;
        document.getElementById("ledDateMode").classList.remove("disabled");
        document.getElementById("ledTempMode").disabled = false;
        document.getElementById("ledTempMode").classList.remove("disabled");
      }

      updateSliderColours();

      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }

    async function getDeviceData_AntipoisoningOptions() {
      
      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 0x07;  // this is the command that requests device LED and tube brightness settings
      
      console.log("Reading device Antipoisoning settings");
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

      if(stat !== 1){
        newReceivedData = false;
        return; // didn't work so data isnt valid
      } 
            
      if(globalReceviedBuffer[2] & 0x01){
        setCheckbox('antiMode', 1);
        window.slideDown( document.getElementById("tubeAntipsnOptions") ); // Show antipoison options
      } else {
        setCheckbox('antiMode', 0); 
        window.slideUp( document.getElementById("tubeAntipsnOptions") ); 
      }
      
      if(globalReceviedBuffer[4] === 0){		// This byte is 1 when antipoisoning=now
        document.getElementById("antiTime").value = (globalReceviedBuffer[2] & 0x06) >> 1; 
      } else {
        document.getElementById("antiTime").value = 4; //Antipsn=now
      }

      document.getElementById("antiPattern").value = (globalReceviedBuffer[2] & 0x18) >> 3;
      document.getElementById("antiLEDDuring").value = (globalReceviedBuffer[2] & 0x20) >> 5;
      document.getElementById("antiLEDAfter").value = (globalReceviedBuffer[2] & 0xC0) >> 6;
      document.getElementById("antiNight").value = (globalReceviedBuffer[5] & 0x0C) >> 2;
        
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }    
    
    // VVV Only for Full Web App VV //
    window.ShowWiFiHelp = function(){
      alert("To connect to your Nixie Clock via WiFi, enter its IP address in the box and click connect. "  
          + "You need a NIX+ model for this to work, and it has to be connected to your home WiFi. "
          + "You can read the IP address from the clock using the buttons, please refer to the quick-start guide or user manual.");
          return;
    }
    
    var _appendBuffer = function(buffer1, buffer2) {
      var tmp = new Uint8Array(buffer1.byteLength + buffer2.byteLength);
      tmp.set(new Uint8Array(buffer1), 0);
      tmp.set(new Uint8Array(buffer2), buffer1.byteLength);
      return tmp.buffer;
    };
    

    async function device_JumpToBootloader() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0xF4;    // Command to jump into bootloader execution space
      
      console.log("Jump to bootloader...");
      await transact_packet(toSendBuffer, false);    // false: no response to be expected

      // Device will disconnect and must be re-connected in bootloader mode
      return;
    } 
    
    
    async function device_JumpToApplication() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0xF3;    // Command to jump into application execution space
      
      console.log("Jump to application...");
      await transact_packet(toSendBuffer, false);    // false: no response to be expected

      // Device will disconnect and must be re-connected in bootloader mode
      return;
    } 
    
    
    async function device_EraseFlash(targetAddr_32, endAddr_32) {
      var CurrentTransfer = 0;
      var TotalTransfers;
      const bytesPerPage = 2048;

      let toSendBuffer = new Uint8Array(64);
      
      window.flashOp_progress = 0;  // Progress percent 0-1
      
      TotalTransfers = ~~((endAddr_32 - targetAddr_32) / bytesPerPage);

      // both addresses must be on a page boundary
      if (((endAddr_32) % bytesPerPage != 0) || (targetAddr_32 % bytesPerPage != 0)) {
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Address error during flash erase. Code 1-' + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer + '"\n');
        alert("Internal address error when when erasing device flash. Code 1-" + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer);          
        return;
      }
      else {

        while (targetAddr_32 < endAddr_32) {

          toSendBuffer[0] = 0xF0;		// Set the USB packet buffer based on the user's inputs
          toSendBuffer[1] = 0;			//0
          toSendBuffer[2] = (targetAddr_32 >> 24) & 0xFF;		// Address MSB. Must be physical address in valid range otherwise you'll get the packet error response (0xFF). erases page that this address falls in
          toSendBuffer[3] = (targetAddr_32 >> 16) & 0xFF;
          toSendBuffer[4] = (targetAddr_32 >> 8) & 0xFF;
          toSendBuffer[5] = (targetAddr_32) & 0xFF;

          console.log("Flash erase operation " + CurrentTransfer + " of " + (TotalTransfers-1));
          window.flashOp_progress = CurrentTransfer/(TotalTransfers-1);  // Progress percent 0-1
          
          let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
          newReceivedData = false;  // data consumed, transact packet is possible again

          if(stat === -1){
            sendHumio('url,'    + window.location.href           + '\n' +
                      'serial,' + window.SerialNumber.toString() + '\n' +
                      'model,'  + window.ModelName.toString()    + '\n' +
                      'message,'+ '"Timeout during flash erase. Code 2-' + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer + '"\n');
            alert("Timeout error while erasing device flash. Code 2-" + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer);          
            return; // didn't work so data isnt valid
          } 

          // Next page to erase
          targetAddr_32 = targetAddr_32 + bytesPerPage;
          CurrentTransfer++;
        }
        
        // erase Completed
        console.log("Flash erase completed successfully");toSendBuffer
        window.flashOp_progress = 1;  // Device flash operation percentage 0-1

        return;
      }
    }
    
    
    // Direct read of device flash memory (encrypted at the device level)
    // Address example: targetAddr_32 = 0x1D00F800 (start of eventlog), endAddr_32 = 0x1D010000 (end of eventlog +1)
    async function device_ReadFlash(targetAddr_32, endAddr_32) {
      var dataArray = new Uint8Array;
      var TotalTransfers;
      var CurrentTransfer;
      var bytesToTransfer;
      const maxBytesPerTransfer = 56;     // Must be 56B
      const byteMultiple = 4;	            // 32bit aligned

      let toSendBuffer = new Uint8Array(64);
      
      window.flashOp_progress = 0;  // Progress percent 0-1
      
      TotalTransfers = ~~((endAddr_32 - targetAddr_32) / maxBytesPerTransfer);
      CurrentTransfer = 0;

      if (((endAddr_32 - targetAddr_32) % maxBytesPerTransfer) != 0) {	// will there be a not-full packet at the end?
        TotalTransfers++;
      }

      // both addresses must be on a page boundary
      if (targetAddr_32 % byteMultiple != 0) {
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Address error during flash read. Code 1-' + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer + '"\n');
        alert("Internal address error when when reading device flash. Code 1-" + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer);          
        return;
      }
      else {

        while (targetAddr_32 < endAddr_32) {

          // Number of bytes to read
          if (~~((endAddr_32 - targetAddr_32) / maxBytesPerTransfer) > 0) {
            // Do a whole transfer
            bytesToTransfer = maxBytesPerTransfer;
          }
          else {
            // Not enough bytes to fill a full transfer
            bytesToTransfer = (endAddr_32 - targetAddr_32);
          }

          toSendBuffer[0] = 0xF1;		                        // Command to read flash
          toSendBuffer[1] = bytesToTransfer;			          // Length in bytes (note: not always the max=60)
          toSendBuffer[2] = (targetAddr_32 >> 24);		      // Address MSB. Must be physical address in valid range otherwise you'll get the packet error response (0xFF)
          toSendBuffer[3] = (targetAddr_32 >> 16);
          toSendBuffer[4] = (targetAddr_32 >> 8);
          toSendBuffer[5] = (targetAddr_32) & 0xFF;
          
          console.log("Reading flash packet " + CurrentTransfer + " of " + (TotalTransfers-1));
          window.flashOp_progress = CurrentTransfer/(TotalTransfers-1);  // Progress percent 0-1
          
          let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
    
          if(stat === -1){
            sendHumio('url,'    + window.location.href           + '\n' +
                      'serial,' + window.SerialNumber.toString() + '\n' +
                      'model,'  + window.ModelName.toString()    + '\n' +
                      'message,'+ '"Timeout during flash read. Code 3-' + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer + '"\n');
            alert("Timeout error while reading device flash. Code 3-" + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer);
            newReceivedData = false;  // data consumed, transact packet is possible again
            return; // didn't work so data isnt valid
          } 

          // It worked, Write the data to the buffer
          dataArray = _appendBuffer(dataArray,globalReceviedBuffer.slice(2,maxBytesPerTransfer+2));
          
          // Next address to read
          if (~~((endAddr_32 - targetAddr_32) / maxBytesPerTransfer) > 0) {
            // Do a whole transfer
            targetAddr_32 = targetAddr_32 + maxBytesPerTransfer;
          }
          else {
            // Not enough bytes to fill a full transfer
            targetAddr_32 = targetAddr_32 + ((endAddr_32 - targetAddr_32) % maxBytesPerTransfer);
          }

          newReceivedData = false;  // data consumed, transact packet is possible again
          CurrentTransfer++;
        }
      }
      // Transfer Completed   
      console.log("Flash read completed successfully");
      window.flashOp_progress = 1;  // Device flash operation percentage 0-1

      return dataArray;
    }
    
    
    async function device_WriteFlash(targetAddr_32, endAddr_32, sourceArray, sourceOffset){
      var CurrentTransfer = 0;
      var TotalTransfers;
      var bytesToTransfer;
      var cpy_ptr;
      var src_ptr = sourceOffset;
      const maxBytesPerTransfer = 56;	// Transmit packet = 64 -2CRC -4ADDR -2INSTR.  Must be multiple of 8
      const WriteMultiple = 8;        // Minimum flash bytes that can be written

      let toSendBuffer = new Uint8Array(64);
      
      window.flashOp_progress = 0;  // Progress percent 0-1
      
      TotalTransfers = ~~((endAddr_32 - targetAddr_32) / maxBytesPerTransfer);

      if (((endAddr_32 - targetAddr_32) % maxBytesPerTransfer) != 0) {	// will there be a not-full packet at the end?
        TotalTransfers++;
      }

      // both addresses must be on a page boundary
      if (((endAddr_32) % WriteMultiple != 0) || (targetAddr_32 % WriteMultiple != 0)) {
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Address error during flash write. Code 1-' + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer + '"\n');
        alert("Internal address error when when writing device flash. Code 1-" + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer);          
        return;
      }
      else {

        while (targetAddr_32 < endAddr_32) {

          // Number of bytes to write
          if (~~((endAddr_32 - targetAddr_32) / maxBytesPerTransfer) > 0) {
            // Do a whole transfer
            bytesToTransfer = maxBytesPerTransfer;
          }
          else {
            // Not enough bytes to fill a full transfer
            bytesToTransfer = (endAddr_32 - targetAddr_32);
            if (bytesToTransfer % WriteMultiple != 0) {
              bytesToTransfer = ~~(bytesToTransfer / WriteMultiple) + 1;	// these bytes will be 0xFF to make up the PIC's required 8 bytes per flash write
              bytesToTransfer = bytesToTransfer * WriteMultiple;		      // round up
            }
          }

          toSendBuffer[0] = 0xF2;		                        // Set the USB packet buffer based on the user's inputs
          toSendBuffer[1] = bytesToTransfer;			          // Length in bytes (note: not always the max=56)
          toSendBuffer[2] = (targetAddr_32 >> 24) & 0xFF;		// Address MSB. Must be physical address in valid range otherwise you'll get the packet error response (0xFF)
          toSendBuffer[3] = (targetAddr_32 >> 16) & 0xFF;
          toSendBuffer[4] = (targetAddr_32 >> 8) & 0xFF;
          toSendBuffer[5] = (targetAddr_32) & 0xFF;
          
          for (cpy_ptr = 0; cpy_ptr < maxBytesPerTransfer; cpy_ptr++) {   // Copy from source array (eg:file) to USB packet
            toSendBuffer[6 + cpy_ptr] = sourceArray[src_ptr];
            src_ptr++;
          }

          console.log("Writing flash packet " + CurrentTransfer + " of " + (TotalTransfers-1));
          window.flashOp_progress = CurrentTransfer/(TotalTransfers-1);  // Progress percent 0-1
          
          let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
          
          if(stat === -1){
            sendHumio('url,'    + window.location.href           + '\n' +
                      'serial,' + window.SerialNumber.toString() + '\n' +
                      'model,'  + window.ModelName.toString()    + '\n' +
                      'message,'+ '"Timeout error during flash write. Code 4-' + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer + '"\n');
            alert("Timeout error while writing device flash. Code 4: " + targetAddr_32 + "-" + endAddr_32 + "-" + CurrentTransfer);          
            newReceivedData = false;  // data consumed, transact packet is possible again
            return; // didn't work so data isnt valid
          } 
          
          // Next address to write
          if (~~((endAddr_32 - targetAddr_32) / maxBytesPerTransfer) > 0) {
            // Do a whole transfer
            targetAddr_32 = targetAddr_32 + maxBytesPerTransfer;
          }
          else {
            // Not enough bytes to fill a full transfer
            targetAddr_32 = targetAddr_32 + ((endAddr_32 - targetAddr_32) % maxBytesPerTransfer);
          }

          newReceivedData = false;  // data consumed, transact packet is possible again
          CurrentTransfer++;
        }
        // Transfer Completed
      }
      
      console.log("Flash write completed successfully");
      window.flashOp_progress = 1;  // Device flash operation percentage 0-1

      return;
    }

    window.ModuleUpdater = async function(forceUpdate){
      var rxStr = "";
      var timeout = 0;
      var itWorked = true;
      var failCnt = 0;
      const MODULE_UPDATE_TIMEOUT = 60; // seconds

      if(!window.HasWiFiModule){
        return false;
      }

      // 0.0.0.0 means not connected so check that too. Don't even attempt update if the module doesnt have an IP
      if( IPpattern.test(document.getElementById("txt_netIP").value) === false || document.getElementById("txt_netIP").value === "0.0.0.0"){
        console.log("Net IP appears invalid. Network connection required to update the WiFi module");
        status("To update WiFi module, a network connection is required");
        return false;
      }

      if( (window.NIXplus_FWver > 0 && window.NIXplus_AppVer > 0) || forceUpdate){
        // We have valid version info..
        if(window.NIXplus_AppVer < window.NIXplus_LatestAppVer || forceUpdate){
          // Update the SPIFFS / WebApp
          console.log("Triggering module SPIFFS update");
          
          rxStr = await writeModule("_UPDATE-SPIFFS", 20);      // Module will reply OK but then be unresponsive until it completes
          
          if(rxStr.slice(0,2) === "OK" ) {
            console.log("Updating, please wait");
            
            for(timeout = 0; timeout < MODULE_UPDATE_TIMEOUT; timeout++){

              rxStr = await writeModule("_APPVERSION?", 5);  // check if completed. If still busy there will be no response.

              if(timeout % 5 === 0 && timeout < 25 && timeout > 0){  // up to 4 progress updates
                fwupg_percent += fwupg_stepsize;
                document.querySelector('#fw_progress').MaterialProgress.setProgress(fwupg_percent);
              }

              if ( rxStr.slice(0,3) === "ERR" ) {
                failCnt++;
                if(failCnt > 3){  // Takes 3 ERR responses before cancelling the operation
                  console.log("Error updating module SPIFFS - operation terminated");
                  sendHumio('url,'    + window.location.href           + '\n' +
                            'serial,' + window.SerialNumber.toString() + '\n' +
                            'model,'  + window.ModelName.toString()    + '\n' +
                            'message,'+ '"Error during SPIFFS update"' + '\n');
                  itWorked = false;
                  break;
                }

              } else if(!isNaN(parseInt(rxStr)) && rxStr > 0 && rxStr != "") {
                window.NIXplus_AppVer = parseInt(rxStr);
                
                if( window.NIXplus_AppVer == window.NIXplus_LatestAppVer){
                  itWorked = true;
                  console.log("Module SPIFFS update completed");
                } else {
                  console.log("SPIFFS update finished but version not latest: " + rxStr);
                  sendHumio('url,'    + window.location.href           + '\n' +
                            'serial,' + window.SerialNumber.toString() + '\n' +
                            'model,'  + window.ModelName.toString()    + '\n' +
                            'message,'+ '"Warning during SPIFFS update - version not the latest: ' + rxStr + '"\n');
                  itWorked = false;
                }
                break;
              }
              await sleep(650);
            }
          } else {
            console.log("Error starting the update, bad or no reply for _UPDATE-SPIFFS");
            itWorked = false;
          }
        }

      failCnt=0;
      if((window.NIXplus_FWver < window.NIXplus_LatestRev || forceUpdate) && itWorked){
        // Update the module firmware
        console.log("Triggering module firmware update");
        fwupg_percent += fwupg_stepsize;

        rxStr = await writeModule("_UPDATE-FLASH", 20);       // Module will reply OK but then be unresponsive until it completes
          
        if(rxStr.slice(0,2) === "OK" ) {
          console.log("Updating, please wait");
          
          for(timeout = 0; timeout < MODULE_UPDATE_TIMEOUT; timeout++){

            rxStr = await writeModule("_FWVERSION?", 5);   // check if completed. If still busy there will be no response. 10=1 second timeout
        
            if(timeout % 5 === 0 && timeout < 25 && timeout > 0){  // up to 4 progress updates
              fwupg_percent += fwupg_stepsize;
              document.querySelector('#fw_progress').MaterialProgress.setProgress(fwupg_percent);
            }

            if ( rxStr.slice(0,3) === "ERR" ) {
              failCnt++;
              if(failCnt > 3){  // Takes 3 ERR responses before cancelling the operation
                console.log("Error updating module firmware - operation terminated");
                sendHumio('url,'    + window.location.href           + '\n' +
                            'serial,' + window.SerialNumber.toString() + '\n' +
                            'model,'  + window.ModelName.toString()    + '\n' +
                            'message,'+ '"Error during WiFi flash update"' + '\n');
                itWorked = false;
                break;
              }

            } else if (!isNaN(parseFloat(rxStr)) && rxStr > 0 && rxStr != "") {
              window.NIXplus_FWver = parseFloat(rxStr);
              // Response is normally blank "" and when it finished updating it will be "0.7" (for example)

                if( window.NIXplus_FWver == window.NIXplus_LatestRev ){
                  itWorked = itWorked & true;
                  console.log("Module flash update completed");
                } else {
                  console.log("Flash update finished but version not latest: " + rxStr);
                  sendHumio('url,'    + window.location.href           + '\n' +
                            'serial,' + window.SerialNumber.toString() + '\n' +
                            'model,'  + window.ModelName.toString()    + '\n' +
                            'message,'+ '"Warning during WiFi flash update - version not the latest: ' + rxStr + '"\n');
                  itWorked = false;
                }
              fwupg_percent += fwupg_stepsize;
              break;
            }
            await sleep(650);
          }
        } else {
          console.log("Error starting the update, bad or no reply for _UPDATE-FLASH");
          itWorked = false;
        }
      }

      await getDeviceInfo();        // refresh all versions including module and update text on app
    }
    if(!itWorked){
      status("Problem updating the WiFi module, power cycle, wait 2 min and try again");
    }
    return itWorked;

  }
    
  async function waitForButtonPress(elementID){
    return new Promise(function(resolve,reject){
      document.querySelector(elementID).addEventListener('click', function() {
        resolve(true);
      });
    });
  }

    window.FirmwareUpdater = async function() {
      const userActivityRequiredTime = 1000;
      var progressString = "";

      if (location.protocol !== 'https:') {
        console.log("Skipped firmware update as protocol is not HTTPS");
        return;
      }

      // Diagnostics updater and connectionChecker interval functions must be stopped
      window.clearInterval(diagUpdater);
      window.clearInterval(connectionChecker);
      diagUpdater = null;
      connectionChecker = null;

      fwupg_percent = 0;
      if( willUpgrade_NIXplus_app==false && willUpgrade_NIXplus_fw==false && willUpgrade_fw==false ){
        if(window.HasWiFiModule){
          fwupg_stepsize = 100/14;  // re-install all three, worth 5 points each, so 6.67% complete per item
        } else {
          fwupg_stepsize = 100 / 5;
        }
      } else {
        fwupg_stepsize = 100 / ( (willUpgrade_NIXplus_app? 5:0) + (willUpgrade_NIXplus_fw? 5:0) + (willUpgrade_fw? 4:0) );
      }

      // first update both NIX+ module fw & app if required. passing true means will force update even though already have latest, otherwise it only updates what is required
      const t0 = performance.now();
      var UpgOk = await ModuleUpdater( willUpgrade_NIXplus_app==false && willUpgrade_NIXplus_fw==false && willUpgrade_fw==false );    

      if( !(willUpgrade_NIXplus_app==false && willUpgrade_NIXplus_fw==false && willUpgrade_fw==false) && !willUpgrade_fw){
        // If its not a complete re-install, and we are not due to update NIX fw, then we are done now
        if(UpgOk){
          document.getElementById("FWupdate_bottomMsg").innerHTML = "Update completed, you can now close this box."
          document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Complete";
          
        } else {
          document.getElementById("FWupdate_bottomMsg").innerHTML = "There was a problem updating the WiFi module, check your WiFi connection then unplug and try again in 2 min."
          document.getElementById("btn_FirmwareUpdate").innerHTML = "Try Again Later";
        }
        document.getElementById("btn_FirmwareUpdate").disabled = true;  // don't want to click it again
        document.getElementById("FWupdate_message").innerHTML = '';
        document.querySelector('#fw_progress').MaterialProgress.setProgress( 100 );
        return;

      } else {
        // We will be updating NIX fw next..
        if(UpgOk){
          document.getElementById("FWupdate_bottomMsg").innerHTML = "Nearly done! Click the button to proceed, then you'll be prompted to re-connect<br><font color='red'>Don't close or refresh your browser.</font>"
        } else if (window.HasWiFiModule) {  // Only say it failed if there was a wifi module there
          document.getElementById("FWupdate_bottomMsg").innerHTML = "There was a problem updating the WiFi module, check your network then unplug and try again in 2 min. Click the button to proceed with the Nixie update anyway (you'll be prompted to re-connect)<br><font color='red'>Don't close or refresh your browser.</font>"
        }
      }

      if(performance.now() - t0 > userActivityRequiredTime){
        // ModuleUpdate() took a while (as it should if doing an update) and so we need user activity to request the USB device later
        // Ask user to interact now = better UX because you click the button (or can cancel) before entering bootloader
        console.log("Requesting user activity (1st time)..");
        
        document.getElementById("btn_FirmwareUpdate").style = "display: none"; 
        document.getElementById("btn_FirmwareUpdateProceed").style = ""; 
        
        await waitForButtonPress("#btn_FirmwareUpdateProceed");

        document.getElementById("btn_FirmwareUpdateProceed").style = "display: none"; 
        document.getElementById("btn_FirmwareUpdate").style = ""; 
      }

      progressString += "Fetching file...";
      console.log("Firmware upgrade step 1/7: Getting firmware file");
      // Step 1: Get the latest firmware for this model type
      try {
        var xhr = new XMLHttpRequest();
        var NowDate = new Date(); // prevents caching
		
        switch (window.ModelName){
          case "DOT6":
            xhr.open('GET',	'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/DOT6_Firmware.hex?v=' + NowDate.getTime(), true);
            break;
            
          case "NIX6":
            xhr.open('GET',	'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/NIX6_Firmware.hex?v=' + NowDate.getTime(), true);
            break;

          case "NIX4":
            xhr.open('GET',	'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/NIX4_Firmware.hex?v=' + NowDate.getTime(), true);
            break;
            
          case "VF4":
            xhr.open('GET',	'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/VF4_Firmware.hex?v=' + NowDate.getTime(), true);
            break;	

          case "FL4":
            xhr.open('GET',	'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/FL4_Firmware.hex?v=' + NowDate.getTime(), true);
            break;  
          
          case "WD1":
            xhr.open('GET',	'https://s3-ap-southeast-2.amazonaws.com/firmware.nixie.com.au/WD1_Firmware.hex?v=' + NowDate.getTime(), true);
            break;

          default:
            //console.error("Error upgrading firmware: unknwon model type!");
            throw "Error upgrading firmware: unknwon model type!"
        }
        
        xhr.responseType = 'arraybuffer';

        xhr.onload = async function() {
          progressString += " File loaded.";
          // response is unsigned 8 bit integer populated upon loading of file

          try {     // Within the [1] try-catch block
            var firmwareBinaryArray = new Uint8Array(this.response); 

            if (this.readyState != 4 || this.status != 200) {
              throw "Error loading the remote firmware file"
            }

            // Step 2: device must enter bootloader mode
            console.log("Firmware upgrade step 2/7: Entering bootloader mode");
            
            if(window.DeviceMode != "BOOTLOADER") {
              progressString += " Jumping to bootloader...";
              await device_JumpToBootloader();  // No need to enter bootlaoder if we're already in it (would give packet-error-response)
              disconnect_tasks();
              await sleep(200);      // Wait for device to re-initialise in bootloader mode

              // The user has to re-connect to the device because it has "physically" disconnect and reconnected
              // If "user active" status expires this try will fail and the user will be asked to interact with the page (button click)
              try {
                await serial.requestPort().then(selectedPort => {
                  port = selectedPort;
                  progressString += " Trying to connect...";
                  connect();
                })
              } catch (e){
                console.log("Requesting user activity (2nd time)..");
                document.getElementById("FWupdate_bottomMsg").innerHTML = "Nearly done! Click the button to proceed, then you'll be prompted to re-connect<br><font color='red'>Don't close or refresh your browser.</font>"
                document.getElementById("btn_FirmwareUpdate").style = "display: none"; 
                document.getElementById("btn_FirmwareUpdateProceed").style = ""; 
                
                progressString += " User activity requested.";
                await waitForButtonPress("#btn_FirmwareUpdateProceed");

                document.getElementById("btn_FirmwareUpdateProceed").style = "display: none"; 
                document.getElementById("btn_FirmwareUpdate").style = ""; 

                await serial.requestPort().then(selectedPort => {
                  port = selectedPort;
                  connect();
                })
              }
        
              await sleep(500);  // connect delay

              if(window.DeviceMode != "BOOTLOADER") {
                console.log("Device did not enter bootloader mode");
                status("Problem entering bootloader, please try again");
                document.getElementById("btn_FirmwareUpdate").disabled = false;
                document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
                progressString += " Didnt enter bootloader!";
                throw "Device did not enter bootloader mode"
              } 
            }

            // Diagnostics updater and connectionChecker interval functions must be stopped
            window.clearInterval(diagUpdater);
            window.clearInterval(connectionChecker);
            diagUpdater = null;
            connectionChecker = null;
        
            console.log("Validating device meets minimum upgrade requirements..");
            progressString += " Validating device...";
            
            // Check that the target device model matches the upgrade model 
            //var file_model = firmwareBinaryArray.slice(0x45, 0x49).join("");
            var model_ptr;
            for(model_ptr=0; model_ptr<window.ModelName.length; model_ptr++){
              if(window.ModelName.charCodeAt(model_ptr) != firmwareBinaryArray[0x45+model_ptr]){
                throw "Device is an incompatible model.";
              }
            }
            
            // Check that the target firmware version meets the upgrade's minimum requirements
            if(window.FW_rev_major < firmwareBinaryArray[0x75]){
              throw "Device has incompatible firmware major version";
            } else if(window.FW_rev_major == firmwareBinaryArray[0x75] && window.FW_rev_minor < firmwareBinaryArray[0x77]){
              throw "Device has incompatible firmware minor version";
            }
            
            // Check that the target firmware version meets the upgrade's minimum requirements
            if(window.BL_rev_major < firmwareBinaryArray[0x65]){
              throw "Device has incompatible bootloader major version";
            } else if(window.BL_rev_major == firmwareBinaryArray[0x65] && window.BL_rev_minor < firmwareBinaryArray[0x67]){
              throw "Device has incompatible bootloader minor version";
            }
            
            // Check that the target hardware version meets the upgrade's minimum requirements
            if(window.HW_rev_major < firmwareBinaryArray[0x55]){
              throw "Device has incompatible hardware major version";
            } else if(window.HW_rev_major == firmwareBinaryArray[0x55] && window.HW_rev_minor < firmwareBinaryArray[0x56]){
              throw "Device has incompatible hardware minor version";
            }

            // Step 3: Erase application flash
            progressString += " Erasing flash...";
            console.log("Firmware upgrade step 3/7: Erasing application flash");
            await device_EraseFlash(0x1D003000, 0x1D00F000);
            fwupg_percent += fwupg_stepsize;
            document.querySelector('#fw_progress').MaterialProgress.setProgress(fwupg_percent);
            
            // Step 4: Write application flash
            progressString += " Writing flash";
            console.log("Firmware upgrade step 4/7: Writing application flash");
            await device_WriteFlash(0x1D003000, 0x1D00F000, firmwareBinaryArray, FIRMWARE_HEADER_SIZE);   
            fwupg_percent += fwupg_stepsize;
            document.querySelector('#fw_progress').MaterialProgress.setProgress(fwupg_percent);
            
            // Step 5: Read application flash back (to verify)
            progressString += " Reading flash...";
            console.log("Firmware upgrade step 5/7: Reading application flash");
            let readData = new Uint8Array(await device_ReadFlash(0x1D003000, 0x1D00F000));
            fwupg_percent += fwupg_stepsize;
            document.querySelector('#fw_progress').MaterialProgress.setProgress(fwupg_percent);
            
            // Step 6: Verify against original binary
            console.log("Firmware upgrade step 6/7: Verifying");
            var cmp_ptr;
            var mismatches = 0;
            var A;
            var B;
            
        // Target length = 0xA000 = 40960
            for(cmp_ptr=0; cmp_ptr < (firmwareBinaryArray.length-FIRMWARE_HEADER_SIZE); cmp_ptr++){
              A = readData[cmp_ptr];
              B = firmwareBinaryArray[cmp_ptr+FIRMWARE_HEADER_SIZE];
              
              if( A != B ){
                // Byte mis-match
                mismatches++;
              }
            }
            
            if(mismatches > 0){
              console.log("Verify failed, " + mismatches + " mismatched bytes");
              document.getElementById("btn_FirmwareUpdate").disabled = false;
              document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
              sendHumio('url,'    + window.location.href           + '\n' +
                        'serial,' + window.SerialNumber.toString() + '\n' +
                        'model,'  + window.ModelName.toString()    + '\n' +
                        'message,'+ '"Error during firmware update - verify failed, mismatches: ' + mismatches + '"\n');
              throw "Verify failed"
            } else {
              progressString += " Verified OK.";
              console.log("Verified OK");
            }
            
            // Step 7: Jump to application space to begin normal execution
            console.log("Firmware upgrade step 7/7: Exiting bootloader mode");
            await device_JumpToApplication();

            disconnect_tasks();

            // Device will now be running in application mode and disconnected from the web app
            
            /*  The code below used to re-connect to the device in app mode but it no longer works in newer chrome, Security Error
            await sleep(1000);      // Wait for device to re-initialise in application mode

            // re-connect
            // The user has to re-connect to the device because it has "physically" disconnect and reconnected
            await serial.requestPort().then(selectedPort => {
              port = selectedPort;
              connect();
            })
                  
            await sleep(500);      // Wait for connect();

            if(window.DeviceMode != "APPLICATION") {
              console.log("Device failed to start in application mode after firmware upgrade");
              status("Problem upgrading firmware, please try again");
            } else {
              console.log("Firmware upgrade completed successfully!");
            }
            */

          } catch (error){
            console.error("Problem upgrading firmware: " + error)
            status("Problem upgrading firmware, please try again");
            document.getElementById("btn_FirmwareUpdate").disabled = false;
            document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
            document.getElementById("FWupdate_bottomMsg").innerHTML = 'Something went wrong, please unplug, reconnect, and try again.'
            sendHumio('url,'    + window.location.href           + '\n' +
                      'serial,' + window.SerialNumber.toString() + '\n' +
                      'model,'  + window.ModelName.toString()    + '\n' +
                      'message,'+ '"Error during firmware update [1]: ' + error + '. Progress string: ' + progressString + '"\n');
            return;
          }

          console.log("Firmware upgrade completed successfully!");
          document.getElementById("btn_FirmwareUpdate").disabled = true;  // don't want to click it again
          document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Complete";
          document.getElementById("FWupdate_message").innerHTML = '';
          document.getElementById("FWupdate_bottomMsg").innerHTML = "Update completed, you can now close this box. To change settings, you'll need to reconnect"
          document.querySelector('#fw_progress').MaterialProgress.setProgress( 100 );
          sendHumio('url,'    + window.location.href           + '\n' +
                    'serial,' + window.SerialNumber.toString() + '\n' +
                    'model,'  + window.ModelName.toString()    + '\n' +
                    'message,'+ '"Firmware updated successfully"' + '\n');
          
        };

        xhr.send();

      } catch (error){
        console.error("Problem upgrading firmware: " + error)
        status("Problem upgrading firmware, please try again");
        document.getElementById("btn_FirmwareUpdate").disabled = false;
        document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
        document.getElementById("FWupdate_bottomMsg").innerHTML = 'Something went wrong, please unplug, reconnect, and try again.'
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during firmware update [2]: ' + error + '. Progress string: ' + progressString + '"\n');
        return;
      }
      
      return;
    }        
    // ^^ Only for full Web App ^^ //
    
    
    window.downloadEventLog = async function() {
      var dataArray = new Uint8Array;

      dataArray = await device_ReadFlash(0x1D00F800, 0x1D010000) // Start of NVM flash eventlog page to end of event log +1
      
      // Transfer Completed   
      console.log("Preparing event log for download");
      console.log(dataArray);

      var datBlob = new Blob([dataArray], {type: "application/octet-stream"});

      // Create a file and download it
      var a = document.createElement("a");
      document.body.appendChild(a);
      a.style = "display: none";

      var url = window.URL.createObjectURL(datBlob);
      a.href = url;
      a.download = "EventLog.hex";
      a.click();
            
      console.log("Download completed");
      sendHumio('url,'    + window.location.href           + '\n' +
                'serial,' + window.SerialNumber.toString() + '\n' +
                'model,'  + window.ModelName.toString()    + '\n' +
                'message,'+ '"Event log downloaded successfully"');

      window.URL.revokeObjectURL(url);                
      
      return;
    }    

    window.CreateSettingsString = async function() {
      // Header Information
      var valStr  = 'filetype,"NIXLabs Settings File V1.23"        \n'
                  + 'timestamp,"'    + this.Date().toString()   + '"\n'
                  + 'model,"'        + window.ModelName         + '"\n'
                  + 'serial,"'       + window.SerialNumber      + '"\n'
                  + 'errorCount,"'   + errCnt                   + '"\n'
                  + 'errorLast,"'    + errLast                  + '"\n'
                  + 'meas5V,"'       + meas5V                   + '"\n'
                  + 'lastThrottle,"' + lastThrottle             + '"\n'
                  + 'throttle,"'     + throttle                 + '"\n'
                  + 'hwversion,"'    + window.HW_rev            + '"\n' 
                  + 'fwversion,"'    + window.FW_rev            + '"\n\n';

      // Settings value. Format is ControlID:Value\n
      // Radio button value must be preceeded by * (handled differently by the reader). Time values must use colon separators
      // Time, Date & Display Options Card
      valStr = valStr + "12hour"                + ",*"+ document.getElementById('12hour').checked.toString()                +"\n";
      valStr = valStr + "24hour"                + ",*"+ document.getElementById('24hour').checked.toString()                +"\n";
      valStr = valStr + "dateFormat_ddmmyy"     + ",*"+ document.getElementById('dateFormat_ddmmyy').checked.toString()     +"\n";
      valStr = valStr + "dateFormat_mmddyy"     + ",*"+ document.getElementById('dateFormat_mmddyy').checked.toString()     +"\n";
      valStr = valStr + "celsius"               + ",*"+ document.getElementById('celsius').checked.toString()               +"\n";
      valStr = valStr + "fahrenheit"            + ",*"+ document.getElementById('fahrenheit').checked.toString()            +"\n";
      valStr = valStr + "leadingZero"           + "," + document.getElementById('leadingZero').checked.toString()           +"\n";
      valStr = valStr + "tail_options_69"       + "," + document.getElementById('tail_options_69').checked.toString()       +"\n";
      valStr = valStr + "tail_options_7"        + "," + document.getElementById('tail_options_7').checked.toString()        +"\n";
      valStr = valStr + "buttonBeep"            + "," + document.getElementById('buttonBeep').checked.toString()            +"\n";
      valStr = valStr + "beepVol_hi"            + ",*"+ document.getElementById('beepVol_hi').checked.toString()            +"\n";
      valStr = valStr + "beepVol_lo"            + ",*"+ document.getElementById('beepVol_lo').checked.toString()            +"\n";
      valStr = valStr + "colonMode"             + "," + document.getElementById('colonMode').value.toString()               +"\n";
      valStr = valStr + "displayMode"           + "," + document.getElementById('displayMode').value.toString()             +"\n";
      valStr = valStr + "showDateEN"            + "," + document.getElementById('showDateEN').checked.toString()            +"\n";
      valStr = valStr + "showTempEN"            + "," + document.getElementById('showTempEN').checked.toString()            +"\n";
      valStr = valStr + "nightThreshMode"       + "," + document.getElementById('nightThreshMode').value.toString()         +"\n";
      valStr = valStr + "nightStart"            + "," + document.getElementById('nightStart').value.toString()              +"\n";
      valStr = valStr + "nightEnd"              + "," + document.getElementById('nightEnd').value.toString()                +"\n";
      valStr = valStr + "enable_tubeAutoBright" + "," + document.getElementById('enable_tubeAutoBright').checked.toString() +"\n";
      valStr = valStr + "enable_ledAutoBright"  + "," + document.getElementById('enable_ledAutoBright').checked.toString()  +"\n";
      valStr = valStr + "displayFlip"           + "," + document.getElementById('displayFlip').checked.toString()           +"\n";
      
      // LEDs & Tube Brightness Card
      valStr = valStr + "sliderRed"         + "," + document.getElementById('sliderRed').value.toString()         +"\n";
      valStr = valStr + "sliderGreen"       + "," + document.getElementById('sliderGreen').value.toString()       +"\n";
      valStr = valStr + "sliderBlue"        + "," + document.getElementById('sliderBlue').value.toString()        +"\n";
      valStr = valStr + "LEDcycler"         + "," + document.getElementById('LEDcycler').value.toString()         +"\n";
      valStr = valStr + "enable_LEDcycler"  + "," + document.getElementById('enable_LEDcycler').checked.toString()+"\n";
      valStr = valStr + "ledTimeMode"       + "," + document.getElementById('ledTimeMode').value.toString()       +"\n";
      valStr = valStr + "brightnessTimeMode"+ "," + document.getElementById('brightnessTimeMode').value.toString()+"\n";
      valStr = valStr + "ledDateMode"       + "," + document.getElementById('ledDateMode').value.toString()       +"\n";
      valStr = valStr + "brightnessDateMode"+ "," + document.getElementById('brightnessDateMode').value.toString()+"\n";
      valStr = valStr + "ledTempMode"       + "," + document.getElementById('ledTempMode').value.toString()       +"\n";
      valStr = valStr + "brightnessTempMode"+ "," + document.getElementById('brightnessTempMode').value.toString()+"\n";
      valStr = valStr + "NightLEDdimming"   + "," + document.getElementById('NightLEDdimming').value.toString()   +"\n";
      valStr = valStr + "TubeDayBrightness" + "," + document.getElementById('TubeDayBrightness').value.toString() +"\n";
      valStr = valStr + "TubeNightBrightness"+"," + document.getElementById('TubeNightBrightness').value.toString()+"\n";
      
      // Antipoisoning Card
      valStr = valStr + "antiMode"          + "," + document.getElementById('antiMode').checked.toString()        +"\n";
      valStr = valStr + "antiPattern"       + "," + document.getElementById('antiPattern').value.toString()       +"\n";
      valStr = valStr + "antiLEDDuring"     + "," + document.getElementById('antiLEDDuring').value.toString()     +"\n";
      valStr = valStr + "antiTime"          + "," + document.getElementById('antiTime').value.toString()          +"\n";
      valStr = valStr + "antiLEDAfter"      + "," + document.getElementById('antiLEDAfter').value.toString()      +"\n";
      valStr = valStr + "antiNight"         + "," + document.getElementById('antiNight').value.toString()         +"\n";
      
      // On-Off Timer Card
      valStr = valStr + "usbWake"           + "," + document.getElementById('usbWake').checked.toString()         +"\n";
      valStr = valStr + "usbSleep"          + "," + document.getElementById('usbSleep').checked.toString()        +"\n";
      valStr = valStr + "resumeLast"        + "," + document.getElementById('resumeLast').value.toString()        +"\n";
      valStr = valStr + "sun1"              + "," + document.getElementById('sun1').checked.toString()            +"\n";
      valStr = valStr + "sun1_on"           + "," +(document.getElementById('sun1_on').value.toString()  ? document.getElementById('sun1_on').value.toString()  : "-")  +"\n";
      valStr = valStr + "sun1_off"          + "," +(document.getElementById('sun1_off').value.toString() ? document.getElementById('sun1_off').value.toString() : "-")  +"\n";
      valStr = valStr + "sun2"              + "," + document.getElementById('sun2').checked.toString()            +"\n";
      valStr = valStr + "sun2_on"           + "," +(document.getElementById('sun2_on').value.toString()  ? document.getElementById('sun2_on').value.toString()  : "-")  +"\n";
      valStr = valStr + "sun2_off"          + "," +(document.getElementById('sun2_off').value.toString() ? document.getElementById('sun2_off').value.toString() : "-")  +"\n"; 
      valStr = valStr + "sat1"              + "," + document.getElementById('sat1').checked.toString()            +"\n";
      valStr = valStr + "sat1_on"           + "," +(document.getElementById('sat1_on').value.toString()  ? document.getElementById('sat1_on').value.toString()  : "-")  +"\n";
      valStr = valStr + "sat1_off"          + "," +(document.getElementById('sat1_off').value.toString() ? document.getElementById('sat1_off').value.toString() : "-")  +"\n";
      valStr = valStr + "sat2"              + "," + document.getElementById('sat2').checked.toString()            +"\n";
      valStr = valStr + "sat2_on"           + "," +(document.getElementById('sat2_on').value.toString()  ? document.getElementById('sat2_on').value.toString()  : "-")  +"\n";
      valStr = valStr + "sat2_off"          + "," +(document.getElementById('sat2_off').value.toString() ? document.getElementById('sat2_off').value.toString() : "-")  +"\n";
      valStr = valStr + "mon1"              + "," + document.getElementById('mon1').checked.toString()            +"\n";
      valStr = valStr + "mon1_on"           + "," +(document.getElementById('mon1_on').value.toString()  ? document.getElementById('mon1_on').value.toString()  : "-")  +"\n";
      valStr = valStr + "mon1_off"          + "," +(document.getElementById('mon1_off').value.toString() ? document.getElementById('mon1_off').value.toString() : "-")  +"\n";
      valStr = valStr + "mon2"              + "," + document.getElementById('mon2').checked.toString()            +"\n";
      valStr = valStr + "mon2_on"           + "," +(document.getElementById('mon2_on').value.toString()  ? document.getElementById('mon2_on').value.toString()  : "-")  +"\n";
      valStr = valStr + "mon2_off"          + "," +(document.getElementById('mon2_off').value.toString() ? document.getElementById('mon2_off').value.toString() : "-")  +"\n";
      valStr = valStr + "tues1"             + "," + document.getElementById('tues1').checked.toString()           +"\n";
      valStr = valStr + "tues1_on"          + "," +(document.getElementById('tues1_on').value.toString() ? document.getElementById('tues1_on').value.toString()  : "-") +"\n";
      valStr = valStr + "tues1_off"         + "," +(document.getElementById('tues1_off').value.toString()? document.getElementById('tues1_off').value.toString() : "-") +"\n";
      valStr = valStr + "tues2"             + "," + document.getElementById('tues2').checked.toString()           +"\n";
      valStr = valStr + "tues2_on"          + "," +(document.getElementById('tues2_on').value.toString() ? document.getElementById('tues2_on').value.toString()  : "-") +"\n";
      valStr = valStr + "tues2_off"         + "," +(document.getElementById('tues2_off').value.toString()? document.getElementById('tues2_off').value.toString() : "-") +"\n";
      valStr = valStr + "wed1"              + "," + document.getElementById('wed1').checked.toString()            +"\n";
      valStr = valStr + "wed1_on"           + "," +(document.getElementById('wed1_on').value.toString()  ? document.getElementById('wed1_on').value.toString()   : "-") +"\n";
      valStr = valStr + "wed1_off"          + "," +(document.getElementById('wed1_off').value.toString() ? document.getElementById('wed1_off').value.toString()  : "-") +"\n";
      valStr = valStr + "wed2"              + "," + document.getElementById('wed2').checked.toString()            +"\n";
      valStr = valStr + "wed2_on"           + "," +(document.getElementById('wed2_on').value.toString()  ? document.getElementById('wed2_on').value.toString()   : "-") +"\n";
      valStr = valStr + "wed2_off"          + "," +(document.getElementById('wed2_off').value.toString() ? document.getElementById('wed2_off').value.toString()  : "-") +"\n";
      valStr = valStr + "thurs1"            + "," + document.getElementById('thurs1').checked.toString()          +"\n";
      valStr = valStr + "thurs1_on"         + "," +(document.getElementById('thurs1_on').value.toString()? document.getElementById('thurs1_on').value.toString() : "-") +"\n";
      valStr = valStr + "thurs1_off"        + "," +(document.getElementById('thurs1_off').value.toString()?document.getElementById('thurs1_off').value.toString(): "-") +"\n";
      valStr = valStr + "thurs2"            + "," + document.getElementById('thurs2').checked.toString()          +"\n";
      valStr = valStr + "thurs2_on"         + "," +(document.getElementById('thurs2_on').value.toString()? document.getElementById('thurs2_on').value.toString() : "-") +"\n";
      valStr = valStr + "thurs2_off"        + "," +(document.getElementById('thurs2_off').value.toString()?document.getElementById('thurs2_off').value.toString(): "-") +"\n";
      valStr = valStr + "fri1"              + "," + document.getElementById('fri1').checked.toString()            +"\n";
      valStr = valStr + "fri1_on"           + "," +(document.getElementById('fri1_on').value.toString()  ? document.getElementById('fri1_on').value.toString()   : "-") +"\n";
      valStr = valStr + "fri1_off"          + "," +(document.getElementById('fri1_off').value.toString() ? document.getElementById('fri1_off').value.toString()  : "-") +"\n";
      valStr = valStr + "fri2"              + "," + document.getElementById('fri2').checked.toString()            +"\n";
      valStr = valStr + "fri2_on"           + "," +(document.getElementById('fri2_on').value.toString()  ? document.getElementById('fri2_on').value.toString()   : "-") +"\n";
      valStr = valStr + "fri2_off"          + "," +(document.getElementById('fri2_off').value.toString() ? document.getElementById('fri2_off').value.toString()  : "-") +"\n";

      // WiFi+BT Module Card. Note, SSIDs and Passwords are not saved.
      valStr = valStr + "wifiEnable"        + "," + document.getElementById('wifiEnable').value.toString()        +"\n";
      valStr = valStr + "wifiTimeSync"      + "," + document.getElementById('wifiTimeSync').value.toString()      +"\n";
      valStr = valStr + "Timezone"          + "," + document.getElementById('Timezone').value.toString()          +"\n";
      valStr = valStr + "use_WiFi_dir"      + "," + document.getElementById('use_WiFi_dir').checked.toString()    +"\n";
      valStr = valStr + "use_WiFi_net"      + "," + document.getElementById('use_WiFi_net').checked.toString()    +"\n";

      // ALS threshold is its own thing and uses a hidden page element. Value is Volts converted to mV/10
      valStr = valStr + "ALSthresh"  + "," + (parseInt(ALSthresh)).toString()                         +"\n";

      valStr = valStr + "End of file";

      return valStr;
    }

    
    window.downloadSettingsFile = async function() {

      var contentStr = await CreateSettingsString();

      const settingsBlob = new Blob( [contentStr], {type : 'text/plain'} );

      // Create a file and download it
      var a = document.createElement("a");
      document.body.appendChild(a);
      a.style = "display: none";

      var url = window.URL.createObjectURL(settingsBlob);
      a.href = url;
      a.download = "NixieSettings.txt";
      a.click();
      window.URL.revokeObjectURL(url);
      
      console.log("Download completed");

      return;
    }    
 
    
    window.restoreSettingsFile = async function(myfiles) {
      var valStr      = "";
      var thisStr     = "";
      var thisControl = "";
      var thisValue   = "";

      if (myfiles.length === 0) {
        console.log('No file was selected');
        return;
      }
      console.log('Reading..');

      var reader = new FileReader();
      reader.readAsText(myfiles[0]);
      
      reader.onload = function(e) {
        
        valStr = reader.result;

        // Ignore the header section
        valStr  = valStr.substr(valStr.indexOf("\n\n")+2, valStr.length-valStr.indexOf("\n\n")-3);
  
        do{
          thisStr = valStr.substr(0, valStr.indexOf('\n'));
          valStr  = valStr.substr(valStr.indexOf('\n')+1, valStr.length-valStr.indexOf('\n'));
  
          thisControl = thisStr.substr(0, thisStr.indexOf(','));
          thisValue   = thisStr.substr(thisStr.indexOf(',')+1, thisStr.length);
  
          if(thisValue === ""){
            //break;

          } else if(thisValue === "true"){
            // true/false means it is a checkbox. need to convert to bool type
            setCheckbox(thisControl, true);

          } else if(thisValue === "false"){
            // true/false means it is a checkbox. need to convert to bool type
            setCheckbox(thisControl, false);

          } else if(thisValue.indexOf(":") != -1){
            // time string (contains colon) so it is a time box which accepts a string
            document.getElementById(thisControl).value = thisValue;

          } else if(thisValue.indexOf("*") != -1){
            // Radio buttons have to use the click() method
            if(thisValue.slice(thisValue.indexOf("*")+1, thisValue.length) === "true"){
              // The radio button is set so click it. the other buttons will become de-selected so we don't care if thisValue === 0
              document.getElementById(thisControl).click();
            }

          } else {
            // all other controls use are set by an integer
            document.getElementById(thisControl).value = parseInt(thisValue);
          }
          
        } while (thisControl != "");

        // Show/hide controls based on value. Not needed for on/off sliders
        if( document.getElementById("displayMode").value == 0){
          document.getElementById('dispModeOptions').style.display = "none";
        } else {
          // auto showing something
          document.getElementById('dispModeOptions').style.display = "";
        }

        if(document.getElementById("nightThreshMode").value == 1){
          // using timer
          document.getElementById('Night_ALSbased').style.display = "none";
          document.getElementById('Night_TimeBased').style.display = "";
          document.getElementById('ALS_slider').style.display = "none";
        } else {
          // Using ALS
          document.getElementById('Night_ALSbased').style.display = "";
          document.getElementById('Night_TimeBased').style.display = "none";
          document.getElementById('ALS_slider').style.display = "";
        }
        updateSliderColours();

        status("Settings file imported OK");
        console.log('Completed!');
  
        return;
      };     
    }  
	
    
    // Dont change any settings just show the custom colour
    window.ShowCustomColour = async function() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 8;    // This is the LED set instruction

      toSendBuffer[2] = document.getElementById("sliderGreen").value;
      toSendBuffer[3] = document.getElementById("sliderRed").value;
      toSendBuffer[4] = document.getElementById("sliderBlue").value;
      toSendBuffer[5] = 1 | 0x80;  // 1= set LEDs to this colour but DONT save to memory, 0x80 = show the custom colour for just 5sec then revert back
        
      console.log("Sending custom colour display request");
      
      let stat = await transact_packet(toSendBuffer);
        
      if(globalReceviedBuffer[2] === 1 && stat===1){
        console.log("LED colour displayed OK");
      }
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }

    // Light show trigger (antipsn round with LED cycle)
    window.lightShow = async function() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0x20;      // This is the special features instruction
      toSendBuffer[0x11] = 3;         // Light show trigger
      
      console.log("Sending the special command");
      
      let stat = await transact_packet(toSendBuffer);
        
      if(globalReceviedBuffer[0] === 0x20 && stat===1){
        console.log("Special command success");
      }
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }

    // Turn the display on or off (just like the on-off timer does)
    // state 0=off, 1=on
    window.dispOnOff = async function(state) {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0] = 0x20;    // This is the special features instruction

      if ( state == 0 ){
        // Turn off
        toSendBuffer[0x10] = 2;
      } else if ( state == 1 ){
        // Turn on
        toSendBuffer[0x10] = 3;
      }
              
      console.log("Sending the special command");
      
      let stat = await transact_packet(toSendBuffer);
        
      if(globalReceviedBuffer[0] === 0x20 && stat===1){
        console.log("Special command success");
      }
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }

    
    // Counter start value, in seconds. Max time is 99h 59m 59s = 359999 seconds.
    // upDown: 0=down, 1=up
    // To cancel an existing timer, set time to 0 and ensure down-count bit is set (no beep will occur).
    // To start an up counter, set ‘Topt’ bit 0 to 1 (with pause bit clear). Up count starts at value in Tmr3-0. 
    // Values > 359999 cause no change to timer (use this to do a read) but note that Topt is still read.
    window.setTimer = async function(seconds, upDown, pause=0, preferTimeScreen=0) {

      if(seconds > 359999){
        // Cant set a timer greater than 99:59:59. But if you send a value >359999 you can use it to read the current timer value
        console.log("Timer cannot exceed 359999 seconds, so reading value only")
      }

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0x20;    // This is the special features instruction
      toSendBuffer[0x01] = (seconds & 0xFF000000) >> 24;
      toSendBuffer[0x02] = (seconds & 0x00FF0000) >> 16;
      toSendBuffer[0x03] = (seconds & 0x0000FF00) >> 8;
      toSendBuffer[0x04] = (seconds & 0x000000FF);

      toSendBuffer[0x05] = (upDown? 1:0) | 0x02 | ((pause? 1:0)<<2) | 0x08 | ((preferTimeScreen? 1:0)<<4);

      if ( state == 0 ){
        // Turn off
        toSendBuffer[0x10] = 2;
      } else if ( state == 1 ){
        // Turn on
        toSendBuffer[0x10] = 3;
      }
              
      console.log("Sending the special command");
      
      let stat = await transact_packet(toSendBuffer);
        
      if(globalReceviedBuffer[0] === 0x20 && stat===1){
        console.log("Special command success");
      }
      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }
    
    
    window.clearSettings = async function() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0x12;    // System special command allows clearing of settings
      toSendBuffer[0x21] = 0x53;    // Control code "S" to erase settings memory

      console.log("Requesting settings memory erase");

      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
        
      if(globalReceviedBuffer[2] & 0x40 && stat===1){
        console.log("Settings erased");
        status("Settings Erased OK"); // All error messages handled by transact packet function

      } else if (stat === -1){
        console.log("Device reported a problem erasing the settings memory");
        status("Error, please try again");
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during clearSettings"' + '\n');
      }

      newReceivedData = false;  // data consumed, transact packet is possible again
      return;
    }    

    // Restart nixie and wifi device (resetNixie=true), and optionally factory reset (erase settings) for nixie device and/or wifi module
    window.resets = async function(resetNixie, eraseNixieSettings, eraseWifiSettings){

      window.clearInterval(diagUpdater);
      window.clearInterval(connectionChecker);
      diagUpdater = null; 
      connectionChecker = null;

      sendHumio('url,'    + window.location.href           + '\n' +
                'serial,' + window.SerialNumber.toString() + '\n' +
                'model,'  + window.ModelName.toString()    + '\n' +
                'message,'+ '"Reset issued"'               + '\n' +
                'resetNixie,'        + resetNixie          + '\n' +
                'eraseNixieSettings,'+ eraseNixieSettings  + '\n' +
                'eraseWifiSettings,' + eraseWifiSettings   + '\n');

      if(eraseWifiSettings && WiFiState != "Feature disabled"){ // If feature is disabled sending WiFi commands will give an error and abort remaining resets
        await issueModuleFactoryReset();  // First we want to erase wifi settings
      }

      if(resetNixie){   // Restart nixie device which will also restart wifi module
        let toSendBuffer = new Uint8Array(64);
        toSendBuffer[0x00] = 0xFA;
        
        if(eraseNixieSettings){
          toSendBuffer[0x02] = 0x42;	// Code to also reset settings to default, it makes the device think MCLR was pressed
        }
        if( await transact_packet(toSendBuffer, false) == 1){
          disconnect_tasks();
        } else {
          console.log("Nixie reset error");
        }
      }

      status("Reset completed! Wait 10s then reconnect if desired");
      console.log("Reset finished");
      return;
    }
    
    
    window.issueModuleFactoryReset = async function(){
      // This reset function is available even if NIX device can't recognise the module
      // First we do a module power cycle and "catch" it in the 1s window between power-up-ready and last-state-resume (in case the problem is the last state)
      if(window.usingWS){
        alert("WiFi module can only be reset via USB. Please connect using USB and try again. Click OK to proceed with the non-WiFi resets..")
        return;
      }

      var result = false;
      result |= await set_WifiModule(MODULE_OFF);   // Disabled moudule in hardware using EN pin 
      await sleep(250);  
      result |= await set_WifiModule(MODULE_ON);    // Enable moudule in hardware using EN pin
      await sleep(500);  
      var rxStr = "";
      rxStr = await writeModule("_FACTORY");      // Send the reset command in the 1s boot-up window.
      
      if(rxStr.slice(0,2) === "OK" && result ) {
        console.log("WiFi Factory reset OK!");
        return true;
      } else {
        console.log("WiFi reset failed or no WiFi module present");
        return false;
      }
    }

    // Find any unicode characters and convert to utf-8 hex values
    function encodeUnicode(s) {
      if(window.usingWS){
        return s; // Websocket transfers complete strings without breaking into bytes, so unicode is handled natively
      } else {
        return unescape( encodeURIComponent(s) );
      }
    }
    
    // Find any utf-8 hex encoded unicode and convert back to symbols
    function decodeUnicode(s) {
      if(window.usingWS){
        return s; // Websocket transfers complete strings without breaking into bytes, so unicode is handled natively
      } else {
        return decodeURIComponent( escape(s));
      }
    }

    // Take the SSID and PW text fields and set the wifi network-connect credentials. 
    // Returns 0 for error, 1= creds not changed, 2= creds changed ok
    async function setWiFiDirectCreds(){

      // User's strings might contain unicode which we have to convert to hex for the module 
      var UCS_SSIDfieldDIR = encodeUnicode(document.getElementById("SSIDfieldDIR").value);  
      var UCS_PWfieldDIR   = encodeUnicode(document.getElementById("PWfieldDIR").value);  

      // SSID has to be at least one character and not more than 32
      if( UCS_SSIDfieldDIR.length > 32){
        console.log("WiFi-Direct SSID entry is too long, limit is 32 characters");
        status("WiFi-Direct SSID too long, limit is 32 characters");
        // document.getElementById("ResponseString").value = "SSID too long, limit is 32";
        return false;
      }
      if( UCS_SSIDfieldDIR.length == 0){
        console.log("WiFi-Direct SSID cannot be blank, hidden networks are not supported");
        status("WiFi-Direct SSID has to be at least 1 character");
        // document.getElementById("ResponseString").value = "SSID cannot be blank";
        return false;
      }

      // Password must be ether blank (none) or at least 8 characters, but not more than 64
      if( UCS_PWfieldDIR.length > 64){
        console.log("WiFi-Direct Password entry is too long, limit is 64 characters");
        status("WiFi-Direct Password too long, limit is 64 characters");
        // document.getElementById("ResponseString").value = "Password too long, limit is 64";
        return false;
      }
      if( document.getElementById("PWfieldDIR").value.length < 8 && document.getElementById("PWfieldDIR").value.length != 0){  // unicode conversion makes 😃=4chars when it should be 2.
        console.log("WiFi-Direct Password entry is too short, can be blank (none) or > 8 characters");
        status("WiFi-Direct Password too short, can be blank or at least 8 characters");
        // document.getElementById("ResponseString").value = "Password too short";
        return false;
      }

      // We now have a valid SSID and PW
      // Check if different to previously stored value, only submit if its different
      if( document.getElementById("SSIDfieldDIR").value == prev_DirSSID && document.getElementById("PWfieldDIR").value == prev_DirPW ){
        console.log("WiFi-Direct SSID and Password not changed, so not submitted");
        return true;
      }

      // We now have values that we know are new and need writing
      var txStr = "WIFIDIRSET\n" + UCS_SSIDfieldDIR;
      if(UCS_PWfieldDIR.length != 0){
        // Also add PW string if there is one. If there isnt one then do not send the additional \n
        txStr = txStr + "\n" + UCS_PWfieldDIR;
      }
      var rxStr = "";
      rxStr = await writeModule(txStr, 50);
      // document.getElementById("ResponseString").value = rxStr;
       
      if (rxStr.slice(0,3) === "ERR"){
        console.log("Problem writing WiFi-Direct credentials");
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during set WiFi-Direct credentials"' + '\n');
        return false;
      } else {
        console.log("WiFi-Direct credentials updated");
        return true;
      }
    }

    window.getWiFiModuleVersion = async function (){
      var rxStrFWVer = "";
      var rxStrAppVer = "";

      window.NIXplus_FWver = 0;
      window.NIXplus_AppVer = 0;  // reset so we know if it worked

      if(!window.HasWiFiModule){
        // no wifi module detected during initialisation
        try {
          document.getElementById("fw_wifiRev").innerText = "N/A";
          document.getElementById("fw_wifiUpgradeRev").innerText = window.NIXplus_LatestRev.toString() + " - " + window.NIXplus_LatestAppVer.toString();
        } catch (err){
          // No problem. Just means diagnostics window not open
        }
        return;
      }

      rxStrFWVer = await writeModule("_FWVERSION?");
      rxStrAppVer = await writeModule("_APPVERSION?");

      window.NIXplus_FWver = parseFloat(rxStrFWVer);  // Format: x.y\n
      window.NIXplus_AppVer = parseInt(rxStrAppVer);  // Format" YYYYMMDD\n

      try {
        document.getElementById("fw_wifiRev").innerText = window.NIXplus_FWver.toString() + " - " + window.NIXplus_AppVer.toString();
        document.getElementById("fw_wifiUpgradeRev").innerText = window.NIXplus_LatestRev.toString() + " - " + window.NIXplus_LatestAppVer.toString();
        
      } catch (err){
        // No problem. Just means diagnostics window not open
      }

      if(window.NIXplus_FWver > 0 && window.NIXplus_AppVer > 0){
        if(window.NIXplus_FWver < window.NIXplus_LatestRev){
          document.getElementById("firmwareStatus").innerHTML = 'There is a NIX+ module update available! <a href="#" id="lnk_openFW" onclick="openFWdialog();">Review changes and install.</a>';
          document.getElementById("FWupdate_message").innerHTML = '';
          document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
          willUpgrade_NIXplus_fw = true;
        }
        if(window.NIXplus_AppVer < window.NIXplus_LatestAppVer){
          document.getElementById("firmwareStatus").innerHTML = 'There is a NIX+ module update available! <a href="#" id="lnk_openFW" onclick="openFWdialog();">Review changes and install.</a>';
          document.getElementById("FWupdate_message").innerHTML = '';
          document.getElementById("btn_FirmwareUpdate").innerHTML = "Update Now";
          willUpgrade_NIXplus_app = true;
        }
      }
      return;
    }

    async function getWiFiModuleStatus(){ 
      var rxStrDir = "";
      var rxStrNet = "";
      var rxStrTZ = "";
      var DirSSID = "";
      var DirPW = "";
      var NetSSID = "";
      var NetPW = "";
      var DirIP = "";
      var NetIP = "";
      var TZstr = "";
      var ADJstr = "";

      if(!window.HasWiFiModule){
        // no wifi module detected during initialisation
        return;
      }

      rxStrDir = await writeModule("WIFIDIRINFO?");
      rxStrNet = await writeModule("WIFINETINFO?");
      rxStrTZ = await writeModule("TIMEZONE?");

      if(rxStrDir.substring(0, 3) === "ERR" || rxStrNet.substring(0, 3) === "ERR" || rxStrTZ.substring(0, 2) != "TZ" ){ // communications error
        console.log("Module returned an error, try again and then restart module if problem persists.");
        status("Problem reading the WiFi module, try again");
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during WiFi module status read"' + '\n');
        return;

      } else if(rxStrDir.substring(0, 3) === "OFF" && rxStrNet.substring(0, 3) === "OFF"){
        setCheckbox("use_WiFi_dir", false);
        setCheckbox("use_WiFi_net", false);
        prev_Dir_EN = false;
        prev_Net_EN = false;
        // "All WiFi is disconnected/off";

      } else if(rxStrDir.substring(0, 3) === "OFF" && rxStrNet.substring(0, 2) === "ON"){
        setCheckbox("use_WiFi_dir", false);
        setCheckbox("use_WiFi_net", true);
        prev_Dir_EN = false;
        prev_Net_EN = true;
        // "WiFi is connected to network";
      
      } else if(rxStrDir.substring(0, 2) === "ON" && rxStrNet.substring(0, 3) === "OFF"){
        setCheckbox("use_WiFi_dir", true);
        setCheckbox("use_WiFi_net", false);
        prev_Dir_EN = true;
        prev_Net_EN = false;
        // "WiFi-Direct is broadcasting";

      } else if(rxStrDir.substring(0, 2) === "ON" && rxStrNet.substring(0, 2) === "ON"){
        setCheckbox("use_WiFi_dir", true);
        setCheckbox("use_WiFi_net", true);
        prev_Dir_EN = true;
        prev_Net_EN = true;
        // "WiFi is connected to network and WiFi-Direct is broadcasting";
      }
    
      // Process response string to get just the components as smaller string to populate app controls
      DirSSID = decodeUnicode( rxStrDir.substring( rxStrDir.indexOf("SSID: ")+6, rxStrDir.indexOf("\nPASS: ")) );   
      NetSSID = decodeUnicode( rxStrNet.substring( rxStrNet.indexOf("SSID: ")+6, rxStrNet.indexOf("\nPASS: ")) );   

      DirPW = decodeUnicode( rxStrDir.substring( rxStrDir.indexOf("PASS: ")+6, rxStrDir.indexOf("\nIP: ")) );   
      NetPW = decodeUnicode( rxStrNet.substring( rxStrNet.indexOf("PASS: ")+6, rxStrNet.indexOf("\nIP: ")) );   

      DirIP = rxStrDir.substring( rxStrDir.indexOf("IP: ")+4, rxStrDir.length); 
      NetIP = rxStrNet.substring( rxStrNet.indexOf("IP: ")+4, rxStrNet.length);   

      TZstr = rxStrTZ.substring(rxStrTZ.indexOf("TZ: ")+4, rxStrTZ.indexOf("\nADJ: "));    // Timezone, eg "Australia/Brisbane"
      ADJstr = parseInt(rxStrTZ.substring(rxStrTZ.indexOf("ADJ: ")+5, rxStrTZ.length));    // Offset adjustment byte +/- 127 min max

      // Populate Text Fields
      document.getElementById("SSIDfieldDIR").value = DirSSID;
      document.getElementById("PWfieldDIR").value = DirPW;
      document.getElementById("txt_dirIP").value = DirIP;
      document.getElementById("SSIDfieldNET").value = NetSSID;
      document.getElementById("PWfieldNET").value = NetPW;
      document.getElementById("txt_netIP").value = NetIP;
      document.getElementById("Timezone").value = TZstr;

      // Keep a local copy of the previous NIX+ credentials so we only re-write if they change
      prev_DirSSID = DirSSID;
      prev_DirPW = DirPW;
      prev_NetSSID = NetSSID;
      prev_NetPW = NetPW;
      prev_TZstr = TZstr; // in hours
      prev_ADJstr = ADJstr;
      // The enabled are done above bit

      // Check if WiFi network is connected
      if( IPpattern.test(document.getElementById("txt_netIP").value) === false || document.getElementById("txt_netIP").value === "0.0.0.0"){
        console.log("Looks like Net IP is invalid. Network connection required to update the WiFi module");
        // We know we have a wifi module to get to here. We also say it will only take 30 sec because the module wont update in its current unconnected state
        document.getElementById("FWupdate_bottomMsg").innerHTML = "<font color='red'>A network connection is required to update the WiFi module - Check your WiFi settings.</font><br>Don't close or refresh your browser. You will be prompted to re-connected to your device once the firmware upgrade starts. The whole process will take less than 30 seconds.<font color='red'><br>Don't close or refresh your browser.</font>";
        // No wifi = no voice assistant. Alert user and disable controls
        document.getElementById("Voice_Msg_space").innerHTML = '<br><font color="red">WiFi-Network is not connected. Please check your WiFi settings.</font>';
        document.getElementById("AWS_Status").innerHTML = ""
        document.getElementById("enableVoiceDiv").style.display = "none";
        document.getElementById("VoiceAcctSetup").disabled = true;

      } else {
        // Valid wifi - proceed to get voice assistant settings
        document.getElementById("VoiceAcctSetup").disabled = false;
        await set_voiceAssistOnOff(true);
      }
      if(document.getElementById("wifiEnable").value != 2){
        document.getElementById("Voice_Msg_space").innerHTML += '<br><font color="red">WiFi must be set to Always On to use Google/Alexa</font>';
      }

      await getWiFiModuleVersion();
      return;
    }

    // Take the timezone entry and write to module if changed
    async function setWifiTimezone(){
      
      var TZnew = document.getElementById("Timezone").value;
      // We now have a valid timezone
      // Check if different to previously stored value, only submit if its different. In hours
      if( TZnew === prev_TZstr ){
        console.log("Timezone unchanged - not submitted");
        return 1;
      }

      // We now have values that we know are new and need writing
      var txStr = "SETTIMEZONE\n" + TZnew + "\n0";
      var rxStr = "";
      rxStr = await writeModule(txStr, 50);

      if (rxStr.slice(0,3) === "ERR"){
        console.log("Problem writing timezone");
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during Set Timezone"'  + '\n');
        //status("Error, please try again");
        return 0;
      } else {
        console.log("Timezone updated");
        //status("Settings saved");
        return 2;
      }
      
    }

    // Take the SSID and PW text fields and set the wifi direct-connect credentials. 
    // Returns 0 for error, 1= creds not changed, 2= creds changed ok
    async function setWiFiNetworkedCreds(){
      
      // User's strings might contain unicode which we have to convert to hex for the module 
      var UCS_SSIDfieldNET = encodeUnicode(document.getElementById("SSIDfieldNET").value);  
      var UCS_PWfieldNET   = encodeUnicode(document.getElementById("PWfieldNET").value);  

      if( document.getElementById("SSIDfieldNET").value == "" && document.getElementById("PWfieldNET").value == "" ){
        console.log("WiFi-Net SSID and Password are default values, so not submitted");
        return true;
      }

      // SSID has to be at least one character and not more than 32
      if( UCS_SSIDfieldNET.length > 32){
        console.log("WiFi-Net SSID entry is too long, limit is 32 characters");
        status("WiFi-Net SSID too long, limit is 32 characters");
        // document.getElementById("ResponseString").value = "SSID too long, limit is 32";
        return false;
      }
      if( UCS_SSIDfieldNET.length == 0){
        console.log("WiFi-Net SSID cannot be blank, hidden networks are not supported");
        status("WiFi-Net SSID has to be at least 1 character");
        // document.getElementById("ResponseString").value = "SSID cannot be blank";
        return false;
      }

      // Password must be ether blank (none) or at least 8 characters, but not more than 64
      if( UCS_PWfieldNET.length > 64){
        console.log("WiFi-Net Password entry is too long, limit is 64 characters");
        status("WiFi-Net Password too long, limit is 64 characters");
        // document.getElementById("ResponseString").value = "Password too long, limit is 64";
        return false;
      }
      if( document.getElementById("PWfieldNET").value.length < 8 && document.getElementById("PWfieldNET").value.length != 0){ // unicode conversion makes 😃=4chars when it should be 2.
        console.log("WiFi-Net Password entry is too short, can be blank (none) or > 8 characters");
        status("WiFi-Net Password too short, can be blank or at least 8 characters");
        // document.getElementById("ResponseString").value = "Password too short";
        return false;
      }

      // We now have a valid SSID and PW
      // Check if different to previously stored value, only submit if its different
      if( document.getElementById("SSIDfieldNET").value == prev_NetSSID && document.getElementById("PWfieldNET").value == prev_NetPW ){
        console.log("WiFi-Net SSID and Password not changed, so not submitted");
        return true;
      }

      // We now have values that we know are new and need writing
      var txStr = "WIFINETSET\n" + UCS_SSIDfieldNET;
      if(UCS_PWfieldNET.length != 0){
        // Also add PW string if there is one. If there isnt one then do not send the additional \n
        txStr = txStr + "\n" + UCS_PWfieldNET;
      }
      var rxStr = "";
      rxStr = await writeModule(txStr, 150); // 15s timeout to wait for a network connection
      // document.getElementById("ResponseString").value = rxStr;
       
      if (rxStr.slice(0,3) === "ERR"){
        console.log("WiFi-Net Problem writing WiFi credentials");   // Problem setting the credentials themselves
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during set WiFi-Net credentials"' + '\n');
        return false;

      } else if(rxStr.slice(0,7) === "Timeout"){                    // WiFi-Net was already on and now can't connect
        console.log("WiFi-Net Failed, check credentials");
        status("Failed to connect to network. Check credentials and retry");
        return true;  // we were able to talk to module

      } else {
        console.log("WiFi-Net WiFi credentials updated");
        return true;
      }
      
    }

    
    // maxDelay is in ms*100 so for a 3sec max delay, maxDelay=30
    // The return string should be parsed for "ERR" as problems with the actual transmission are signified by a debug string starting with "ERROR"
    window.writeModule = async function(txStr, maxDelay=10) {
      
      var expecingResponse = false;
      let toSendBuffer = new Uint8Array(64);
      
      var p_toSendBuffer = 0;
      var p_TXstr = 0;
      var timeout = 0;
      var rxStr = "";

      if(txStr.length > 180){ // Max size is 3x60=180 Bytes
        console.log("Maximum message size exceeded (" + txStr.length + "/180)");
        return "ERROR TX Size";
      }

      console.log("Write module: " + txStr);

      if( window.usingWS ) {
        // Using websocket so we use a different command to write to the module directly
        var setOK;
        var retryCount = 0;
        do{
          retryCount++;
          setOK = false;
          timeout = 0;

          while(window.portBusy && timeout < BUSY_WAIT_TIMEOUT_MS){   // Wait for the interface to be free, otherwise we abort (dont want to send multiple things at once to the device)
            await sleep(1)
            timeout++;
          }
          if(timeout >= BUSY_WAIT_TIMEOUT_MS){
            console.log("Error TX. Timed out waiting for port to be free [WS]");
            //status("Port busy, reload page and try again.");
            return "ERROR TX Port Busy [WS]";
          }

          moduleDirectReadPending = true;
          ws.send( "\uFEFE" + txStr);  // add three special bytes (0xEF,BB,BE) to tell the module this is a direct write
          // When the WS receives the response, the readPending flag is cleared
          while(moduleDirectReadPending && timeout < maxDelay){
            await sleep(100); 
            timeout++;
          }

          window.portBusy = false;  // reset, ready for next transaction

          if(timeout >= maxDelay) {
            console.log("Timed out waiting for module response [WS]");
            setOK = false;

          } else if ( WsRXdata.slice(0,3) === "ERR" ){
            console.log("Module returned an error [WS]");
            setOK = false;

          } else {
            if(window.packet_data_debug){
              console.log("Module Message Sent [WS]");
            }
            setOK = true;
          }
        } while(!setOK && retryCount < maxPacketRetries);
  
        if(setOK){
          return WsRXdata;  // populated by the WS event listener
        } else {
          return "ERROR Module Timeout or ERR [WS]";   // No point in processing this invalid packet
        }

      } else {
        // Using USB via NIX device
        while( (p_toSendBuffer + p_TXstr*60) < txStr.length ){

          for(p_toSendBuffer=0; p_toSendBuffer < 60; p_toSendBuffer++) {   // Max length=60 bytes per transfer, if you want null terminators and things, you have to send them!
            if( (p_toSendBuffer + p_TXstr*60) > txStr.length){
              // We have reached the end of the message
              toSendBuffer[0x01] = p_toSendBuffer | 0x80;   // message length fits into one packet. clear rx buffer (0x80)
              expecingResponse = true;                      // A complete message has been sent so we will look for a response
              break;

            } else {
              toSendBuffer[0x01] = 60 | 0x80;           // bit x80 clears the module's response buffer, we will keep clearing it until we have finished sending.
              expecingResponse = false;                 // There is more to send and the module wont respond until we have sent it all
            }
            
            toSendBuffer[0x00] = 0xE2;                  // System special command allows direct write to module UART
            toSendBuffer[p_toSendBuffer+2] = txStr.charCodeAt(p_toSendBuffer + p_TXstr*60);
          }

          let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
          
          newReceivedData = false;  // data consumed, transact packet is possible again

          if(globalReceviedBuffer[0x01] === 0 && stat===1){
            console.log("Message sent (block " + p_TXstr + ")");

          } else if(globalReceviedBuffer[0x01] > 0 && stat===1){
            console.log("Device reported a problem writing to the UART (block " + p_TXstr + ") - Module Busy");
            return "ERROR Module busy";   // No point in processing this invalid packet

          } else {
            console.log("Transact packet failed");
            return "ERROR Module Timeout or ERR";   // No point in processing this invalid packet
          }
          
          p_toSendBuffer = 0;
          p_TXstr++;  // next block
        }

        if(expecingResponse){
          let toSendBuffer2 = new Uint8Array(64);

          // We must allow time for module to respond. This could take a while for some tasks, so we query the availabe bytes value
          // get function Returns -1 for error, 0 for completed OK no data in module RX buffer. >0 if data is ready for reading in RX buffer (value is number of bytes)
          timeout = 0;
          var nowBytes = 0;
          var prevBytes = 0;

          console.log("Checking for module response..");
          while (timeout < maxDelay && (port || window.usingWS)){ // keep checking unless both USB and WS are closed
            await sleep(50);
            prevBytes = await wifiModule_BytesReady();
            await sleep(50);
            nowBytes = await wifiModule_BytesReady();

            if( prevBytes === nowBytes && nowBytes > 0 ){
              // Two consecutive non-zero byte readings means the data transfer is done
              break;
            }

            // Special case if waiting for "a while" to connect to wifi, to let the user know what's happening
            if(timeout === 25){  // after 2.5 seconds. Normally WIFINET takes ~ 2sec
              status("Processing, please wait..");
            }
            if(timeout === 60){  // 6sec. Failed wifi connection takes 10s to time out
              status("Still working...");
            }
            timeout++;
          }

          var block;

          for(block=0; block<3; block++) {  // Read all blocks (180 byte buffer = 3 blocks of 60 bytes). Unwritted bytes are zero
            toSendBuffer2[0x00] = 0xE3;     // Read the module response
            toSendBuffer2[0x01] = block;

            let stat = await transact_packet(toSendBuffer2);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

            newReceivedData = false;  // reset, ready for next transaction

            if(stat !== 1){
              console.log("Error during response receive (block " + block + ")");
              return "ERROR RX Timeout";   // No point in processing this invalid packet
            }

            var i;
            for(i=0; i<60; i++){
              if(globalReceviedBuffer[i+2] != 0){   // Null characters (0) are not printed, although they could be valid for device-level transactions (not human readable anyway)
                rxStr = rxStr + String.fromCharCode(globalReceviedBuffer[i+2]);
              }
            }
          }

          // Convert to human readable string. 
          console.log("Module Response: ");
          console.log(rxStr);
        }
    }

      return rxStr;
    }    

    
    window.clearEventLog = async function() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0x12;    // System special command allows clearing of event log
      toSendBuffer[0x20] = 0x45;    // Control code "E" to erase event log
      
      console.log("Requesting event log erase");
      
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
      
      newReceivedData = false;  // data consumed, transact packet is possible again
      
      if(globalReceviedBuffer[0x02] & 0x80 && stat===1){
        console.log("Event log erased");
        status("Event Log Erased OK");
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Event log cleared"' + '\n');
      } else if(stat === -1){
        console.log("Device reported a problem erasing the event log memory. Code " + globalReceviedBuffer[2] + "-" + globalReceviedBuffer[3]);
      }

      return;
    }    

    window.setSpecial = async function() {

      let toSendBuffer = new Uint8Array(64);
      
      toSendBuffer[0x00] = 0x20;    // Special functions command
      toSendBuffer[0x01] = 0x00;    // Tmr3
      toSendBuffer[0x02] = 0x00;    // Tmr2
      toSendBuffer[0x03] = 0x00;    // Tmr1
      toSendBuffer[0x04] = 0x00;    // Tmr0
      toSendBuffer[0x05] = 0x00;    // Timer Options

      toSendBuffer[0x10] = 0x00;    // Power control
      toSendBuffer[0x11] = 0x00;    // Screen control
      
      console.log("Setting special function command");
      
      let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
      
      newReceivedData = false;  // data consumed, transact packet is possible again
      
      if(globalReceviedBuffer[0x00] === 0x20){
        console.log("OK");
        status("OK");
      } else if(stat === -1){
        console.log("Device reported a problem. Code " + globalReceviedBuffer[0] + "-" + globalReceviedBuffer[1]);
        sendHumio('url,'    + window.location.href           + '\n' +
                  'serial,' + window.SerialNumber.toString() + '\n' +
                  'model,'  + window.ModelName.toString()    + '\n' +
                  'message,'+ '"Error during special function command. Code ' + globalReceviedBuffer[0] + "-" + globalReceviedBuffer[1] + '"\n');
      }

      return;
    }  
    

// ESP32 WIFI+BT Module read Status
// showWiFiPendingMsg: true: use for first settings read, if module is initialising it will show a please refresh page snackbar. 
// use false for Diagnostics read because otherwise that message is not useful and annoying.
window.getDeviceData_WifiModule = async function(showWiFiPendingMsg = true) {

  let toSendBuffer = new Uint8Array(64);
  
  toSendBuffer[0] = 0xE0;  // this is the command that requests device ESP32 module status
  
  console.log("Reading device WiFi settings");

  let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

  var statusStr = "";
  if(stat === 1){
    // No snackbar for a settings read, but we do update the diagnostic information
    
    // Note: WiFiState is also used globally
    switch(globalReceviedBuffer[0x01]){
      case 0: 
          WiFiState = "Feature disabled"; // Note: This string is looked for elsewhere
          break;
      case 1:
          WiFiState = "Not detected";
          break;
      case 2:
          WiFiState = "Init Pending";
          break;    
      case 3:
          WiFiState = "Init Connecting";
          break;
      case 4:
          WiFiState = "Init Rx Pending";
          break;
      case 6:
          WiFiState = "Active";
          break;        
      case 7:
          WiFiState = "Active, Time Sync";
          break;            
      case 8:
          WiFiState = "Active, Time Synched";
          break;          
      case 9:
          WiFiState = "Suspended";
          break;    
      case 10:
          WiFiState = "Preparing to suspend";
          break;     
      case 11:
          WiFiState = "Active, Fetching IP";
          break;    
      case 12:
          WiFiState = "Active, Direct Comm";
          break;    
      case 13:
          WiFiState = "Suspend pending";
          break;    
      case 14:
          WiFiState = "Present, Init Pending";
          break;    
      case 15:
          WiFiState = "Present, Init RX Pending";
          break;    
      default:
          WiFiState = "Undefined State";
          break; 
    }
    //          State Str                  State Num                          RX bytes                           TX bytes                    EN pin (1=en, 2=disabled)   
    statusStr = WiFiState + " (" + globalReceviedBuffer[0x01] + ") " + globalReceviedBuffer[0x02] + "-" + globalReceviedBuffer[0x03] + "-" + globalReceviedBuffer[0x05];
    
    if(globalReceviedBuffer[0x06] === 2){  // flag for module disabled due to throttling
      statusStr += "<br>USB voltage too low for WiFi!";
    }
  } else {
    statusStr = "Unknown, read failed";
    newReceivedData = false;  // data consumed, transact packet is possible again
    return;
  }
        
  // Ready module status (0x01) and isThrottling code (0x06)
  // if module is off due to throttling we can't tell if it is there or not so dont show any message
  if(globalReceviedBuffer[0x01] >= 5){          // Module is connected and finished initialising. Module state machine state, refer to communications specificaiton
    window.HasWiFiModule = true;
    if( document.getElementById("statusText").innerHTML.indexOf("+") === -1 ){
      // Model name should have a "+" to show it has WiFi, append it if it doesnt already exist
      document.getElementById("statusText").innerHTML = window.ModelName + "+ Connected";
    }

  } else if (globalReceviedBuffer[0x01] === 2 && globalReceviedBuffer[0x06] != 2){  // Module is still initialising so not sure yet if present or not
    window.HasWiFiModule = false;

    if(showWiFiPendingMsg){
      status("WiFi-Module detection pending, please refresh page");
      console.log("WiFi module is still initialising, assuming there is none. Refresh page in a few seconds to retry");
    }
  } else {
    window.HasWiFiModule = false;         // there is no wifi module attached, it hasn't started due to throttling, or the feature is disabled
  }

  if(globalReceviedBuffer[0x06] === 2){
    window.WiFiOffDueToThrottling = true;
    document.getElementById("WiFi_Msg_space").innerHTML = '<br><font color="red">USB voltage is too low for WiFi! Try a different power adaptor and avoid unpowered USB hubs.  WiFi is currently suspended but you can still make settings changes.</font>';
    document.getElementById("Voice_Msg_space").innerHTML = '<br><font color="red">USB voltage is too low for WiFi! WiFi is currently suspended but you can still make settings changes.</font>';
  } else {
    window.WiFiOffDueToThrottling = false;
    document.getElementById("WiFi_Msg_space").innerHTML ='';
    document.getElementById("Voice_Msg_space").innerHTML ='';
  }

  // Hide/show the WiFi settings card based on model
  if(showWiFiPendingMsg){
    if(window.HasWiFiModule){
      document.getElementById('cardWifi').style.display = "block";	// show 
      document.getElementById('cardVoiceAssistant').style.display = "block";
    } else {
      document.getElementById('cardWifi').style.display = "none";	  // hide
      document.getElementById('cardVoiceAssistant').style.display = "none";
    }
  }

  document.getElementById("wifiTimeSync").value = (globalReceviedBuffer[0x04] & 0x06) >> 1;   // Config settings -- time sync 0-2

  if(globalReceviedBuffer[0x04] & 0x10){
    // module disabled fully
    document.getElementById("wifiEnable").value = 0;

  } else if(globalReceviedBuffer[0x04] & 0x01){
    // module always enabled
    document.getElementById("wifiEnable").value = 2;

  } else {
    // module enabled for time sync only
    document.getElementById("wifiEnable").value = 1;
  }

  try {  // The diag_ fields are null until the diagnostics window has been loaded.
    if(window.HasWiFiModule === true){
      document.getElementById("diag_ModelName").innerText =  window.ModelName + "+"; 
    } else {
      document.getElementById("diag_ModelName").innerText =  window.ModelName;
    }
    document.getElementById("diag_wifi").innerHTML = statusStr;

  } catch(err){
    // No problem. Just means diagnostics window not open
  }

  newReceivedData = false;  // data consumed, transact packet is possible again
  return;
}

// Returns -1 for error, 0 for completed OK no data in module RX buffer. >0 if data is ready for reading in RX buffer (value is number of bytes)
async function wifiModule_BytesReady() {
      
  let toSendBuffer = new Uint8Array(64);
  
  toSendBuffer[0] = 0xE0;  // this is the command that requests device ESP32 module status
  
  //console.log("Reading available module RX bytes");

  let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
  newReceivedData = false;  // data consumed, transact packet is possible again

  if(stat !== 1){
    return -1; // didn't work so data isnt valid
  } else if(globalReceviedBuffer[0x02] > 0){
    return globalReceviedBuffer[0x02];  // There is data in the RX buffer. Caller should ensure it is complete by calling this function twice.
  }

  return 0;
}


// ESP32 WIFI+BT Module set mode
submitWifi.addEventListener('click', set_allWifi);  
async function set_allWifi(){
  var result = true;
  var TZresult;

  // Fairly slow function because of having to talk to the module via the host clock. so don't allow repeated sends until completed.
  document.getElementById("submitWifi").disabled = true;

  // Send the settings file to Humio to help improve NIXlabs products. We don't share this info with anyone else - see our Privacy Policy
  var contentStr = await CreateSettingsString();
  sendHumio('url,'    + window.location.href            + '\n' +
            'message,'+ '"Submitted the WiFi settings"' + '\n' + contentStr);

  TZresult = await setWifiTimezone();
  TZresult == 0? result = false : result = true;
  result &= await setWiFiDirectCreds();     // Set SSIDs and PWs, they wont be re-written if they havent changed. module automatically re-boots wifi if its on and the creds change
  result &= await setWiFiNetworkedCreds();  // If wifiNet already on, this could take up to 10-15s then give status msg saying check credentials
  result &= await set_WifiModule();       

  var timeout = 0; 

  // If wepapp has set wifi to any kind of 'on'
  for(var i=0; i<2; i++){ // have to check twice because in between starting up and GETTIME there will be a brief period of Active state
    do {
      // Wait until module has finished initialising
      await sleep (250);  // No need to hammer it. Typ reconnect time around 4s. Module's timeout value is 10s
      await getDeviceData_WifiModule(false);       // Check module status. false=dont show snackbar messages
      timeout++;

      // Special case if waiting for "a while" to connect to wifi, to let the user know what's happening
      if(timeout === 10){  // after 2.5 seconds. Normally WIFINET takes ~ 2sec
        status("Connecting to WiFi Network, please wait..");
      }

    } while( !(WiFiState === "Active" || WiFiState === "Suspended") && (timeout < MODULE_WIFICONNECT_TIMEOUT_250MS) );
  } // note: timeout not cleared between for() loops so total time is limited to the timeout value (15s)


  // Enable wifi net/dir. WIFIDIR must come before WIFINET, else WIFINET loses the IP. Just enabling WIFIDIR is OK because the module check for this state
  if ( (prev_Dir_EN && document.getElementById("use_WiFi_dir").checked === false) || (prev_Net_EN && document.getElementById("use_WiFi_net").checked === false) ){
    await writeModule("WIFIOFF"); // have to turn off all wifi even to turn off just one mode
    prev_Dir_EN = false;
    prev_Net_EN = false;

    if( document.getElementById("use_WiFi_dir").checked === true ){
      await writeModule("WIFIDIR", 50);     // doesn't matter what it was before, turn on WiFi direct
      prev_Dir_EN = true;
    }
    if ( document.getElementById("use_WiFi_net").checked === true ){
      netRxStr = await writeModule("WIFINET", MODULE_WIFICONNECT_TIMEOUT_250MS*2.5);  // turn on WiFi network if it was off before. Wait up to 15s as ESP32 connection timeout is 10s
      if(netRxStr.slice(0,7) === "Timeout") {
        status("Failed to connect to network. Check credentials and retry");
        prev_Net_EN = false;  // The module keeps WiFi Net off if it times out
      }
      prev_Net_EN = true;
    }

  } else {
    if( !prev_Dir_EN && document.getElementById("use_WiFi_dir").checked === true ){
      await writeModule("WIFIDIR", 50);   // turn on WiFi direct if it was off before
      prev_Dir_EN = true;
    }
    if ( !prev_Net_EN && document.getElementById("use_WiFi_net").checked === true ){
      netRxStr = await writeModule("WIFINET", MODULE_WIFICONNECT_TIMEOUT_250MS*2.5);  // turn on WiFi network if it was off before. Wait up to 15s as ESP32 connection timeout is 10s
      if(netRxStr.slice(0,7) === "Timeout") { 
        status("Failed to connect to network. Check credentials and retry");
        prev_Net_EN = false;  // The module keeps WiFi Net off if it times out
      }
      prev_Net_EN = true;
    }
  }

  if(result){

    await sleep(500);

    await getDeviceData_WifiModule(false);  // Check module status. false=dont show snackbar messages. This call must come before getWiFiModuleStatus
    await getWiFiModuleStatus();            // read everything from module in order to re-populate IP addresses mainly but also as a check
    
    // Now that we are done with the wifi module, tell the clock to fetch a new time if the timezone changed and time sync is enabled
    if( TZresult == 2 && document.getElementById("wifiTimeSync").value != 0 ){
      forceTimeUpdate();
    }

    // Update the previous values so we can detect a change and know when to submit
    prev_DirSSID = document.getElementById("SSIDfieldDIR").value;
    prev_DirPW = document.getElementById("PWfieldDIR").value;
    prev_NetSSID = document.getElementById("SSIDfieldNET").value;
    prev_NetPW = document.getElementById("PWfieldNET").value;
    prev_TZstr = document.getElementById("Timezone").value;
    //prev_ADJstr = document.getElementById("ADJstr").value;
    
    status("Settings saved");

    // Alert user to low wifi voltage, if the user wants wifi on
    if(window.WiFiOffDueToThrottling && (prev_Net_EN || prev_Dir_EN) ){
      status("USB voltage too low to enable WiFi!");
    }

  } else {
    // dont repopulate fields if something went wrong
    status("Error, please try again");
  }

  document.getElementById("submitWifi").disabled = false; // allow sending again

  return;
}

async function set_WifiModule(EN_pin_state=0) {

  let toSendBuffer = new Uint8Array(64);
  
  toSendBuffer[0] = 0xE1;   // this is the command that set module operating settings
  toSendBuffer[5] = 1;      // Start by setting the 'apply settings' bit

  if(document.getElementById("wifiEnable").value == 0){
    // WiFi Disabled bit
    toSendBuffer[2] |= 0x10;
  } else if(document.getElementById("wifiEnable").value == 2){
    // Always on bit
    toSendBuffer[2] |= 0x01;
  } // else, we chose only on for time sync, which is no bit set

  toSendBuffer[3] = EN_pin_state;   // 0=no change, 1=module on, 2=module off

  toSendBuffer[2] |= (document.getElementById("wifiTimeSync").value << 1);

  console.log("Writing WiFi settings");

  let stat = await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries

  if(stat === 1){
    console.log("WiFi preferences were set OK");
    //status("Settings saved");
    return true;  // worked

  } else if (stat === -1){
    console.log("Error during setting write, perhaps WiFi feature not supported by your model");
    //status("Error, check you have a NIX+ model and try again");
    return false;  // failed
  }

}

async function forceTimeUpdate() {

  let toSendBuffer = new Uint8Array(64);
  
  toSendBuffer[0] = 0xE1;   // this is the command that set module operating settings
  toSendBuffer[4] = 1;      // force time update

  console.log("Forcing time update..");

  await transact_packet(toSendBuffer);    // Does everything including CRC, port-free check, data read-back and integrity/timeout check plus retries
  return;
}

document.getElementById("enableVoice").addEventListener('change', function() {set_voiceAssistOnOff(false);} );  
async function set_voiceAssistOnOff(initOnly=false){

  var shouldBeOn = false; // Should AWSSTATE? be "Connected" (true) or might it be off (false)?
  
  // Make sure the module is ready to receive commands that may change MQTT state
  var rxStrAWSstat = "Pending"; // To allow thw while() to run at least once
  var timeout = 0;
  document.getElementById("AWS_Status").innerHTML = "Please wait"
  document.getElementById("AWS_Status_Spinner").style.display = "block";

  // There will be a point where the module can talk back but it might not be connected yet..
  while( timeout < AWS_CONNECT_TIMEOUT_500MS && (port || window.usingWS) && rxStrAWSstat.slice(0,7) === "Pending"){
    rxStrAWSstat = await writeModule("AWSSTATE?");
    await sleep(350);
    timeout++;
  }
  if(timeout >= AWS_CONNECT_TIMEOUT_500MS){
    console.log("Timed out waiting for AWSSTATE to finish (1)");
    status("RX error, disconnect and try again");
    
    return;
  }

  // readCheckbox = false to just read module and display parameters. True= normal case to read the switch and take action if required
  if(!initOnly && document.getElementById("enableVoice").checked === true){
    var rxStr = await writeModule("VOICEASSIST ON");  
      if(rxStr.slice(0,2) !== "OK") {
        status("RX error, disconnect and try again");
      } else {
        shouldBeOn = true;
      }
  } else if(!initOnly && document.getElementById("enableVoice").checked === false){
    var rxStr = await writeModule("VOICEASSIST OFF");  
      if(rxStr.slice(0,2) !== "OK") {
        status("RX error, disconnect and try again");
      }
  } 
  
  if(initOnly){
    const guidPattern = new RegExp(/[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}/);
    if( guidPattern.test(window.userSub) ){
      // The user has just logged in and got a valid sub so use it (even if its already correct, because you only get the sub if the user actively logged in not always)
      //console.log("User sub = " + window.userSub);
      await writeModule("AWSIOT_SETUP " + window.userSub + " 0"); // set id, device number 0
      await writeModule("_AWSRECERT");  // establish AWS certificate
      await sleep(5000);  // Needs some time to get the serial from the host device and commence AWS connection (better to not interrupt else no serial)
      shouldBeOn = true;
      await writeModule("VOICEASSIST ON");
    }
  }

  // get the latest AWS connection status
  rxStrAWSstat = "Pending"; // To allow thw while() to run at least once
  timeout = 0;
  // There will be a point where the module can talk back but it might not be connected yet..
  while( timeout < AWS_CONNECT_TIMEOUT_500MS && (port || window.usingWS) && (rxStrAWSstat.slice(0,7) === "Pending" || (shouldBeOn && rxStrAWSstat.slice(0,9) !== "Connected")) ){
    rxStrAWSstat = await writeModule("AWSSTATE?");
    await sleep(350);
    timeout++;
  }
  if(timeout >= AWS_CONNECT_TIMEOUT_500MS){
    console.log("Timed out waiting for AWSSTATE to finish (2)");
  }

  document.getElementById("AWS_Status_Spinner").style.display = "none";
  var rxStrAWSid = await writeModule("_CERTID?"); 
  var rxIotName = await writeModule("_IOTNAME?");

  document.getElementById("AWS_ID").innerHTML = rxStrAWSid;
  document.getElementById("AWS_Status").innerHTML = rxStrAWSstat;
  document.getElementById("AWS_iotName").innerHTML = rxIotName;

  const idPattern = new RegExp(/[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}\_[0-9]/);

  if( idPattern.test(rxIotName) ){
    // Module already has a valid device ID.
    document.getElementById("VoiceAcctSetup").innerHTML = "Relink Account";
    document.getElementById("VoiceAcctSetup").classList.remove("mdl-button--accent");
    document.getElementById("enableVoiceDiv").style.display = "block";
  } else {
    // Module not set up
    if(window.userSub != "")
    document.getElementById("VoiceAcctSetup").innerHTML = "Create Account or Login"; 
    document.getElementById("VoiceAcctSetup").classList.add("mdl-button--accent");
    document.getElementById("enableVoiceDiv").style.display = "none"; 
  }

  if(rxStrAWSstat.slice(0,9) === "Connected"){
    setCheckbox("enableVoice", true);
  } else {
    setCheckbox("enableVoice", false);
  }

  return;
}

// Does everything including CRC, port-free check, data read-back and integrity check
// The data packet is card-specific and constructed in the card's click handler function
// Return -2 for fatal port error, -1 for data not available, 1 for successful send and receive
// Caller function must clear newReceivedData when data is consumed
async function transact_packet(toSendBuffer, expectResponse = true){
  var crc;
  var timeout;
	
  timeout = 0;
  // Wait for the interface to be free, and previous data to be consumed 
  while((window.portBusy || newReceivedData) && timeout < BUSY_WAIT_TIMEOUT_MS){   
    await sleep(1)
    timeout++;
  }
  //console.log("waited " + timeout);
  if(!window.portBusy && newReceivedData && timeout >= BUSY_WAIT_TIMEOUT_MS){
    // Caller did not consume the data. Probably doesnt want it anymore so carry on

  } else if(timeout >= BUSY_WAIT_TIMEOUT_MS){
    console.log("Timed out waiting for port to be free");
    status("Port busy error, please unplug and try again");
    disconnect_tasks();
    return -2;
  }
  timeout = 0;

  if (!port && !window.usingWS) {  // check if the port is open and exists
    console.log("Port closed or does not exist!");
    status("Port closed error, please unplug and try again");
    disconnect_tasks();
    return -2;
  }

  // Now calc the checksum of bytes 0->62, and put the result in bytes 62 and 63
  crc = crc16(toSendBuffer, 62);
  // Since we are dealing with bytes, must convert the CRC (two bytes) into seperate bytes 
  toSendBuffer[62] = crc >> 8;  // shift the 16bit value to the right by 8 bits, so (hex) 0x1234 becomes 0x12
  toSendBuffer[63] = crc & 0xFF;  // mask off the upper byte so that 0x1234 becomes 0x34
  if(window.packet_data_debug){
    console.log(toSendBuffer);
  }

  window.portBusy = true; 

  let retryCount=0;
  let maxWaitTime;
  if(window.usingWS){
    maxWaitTime = WS_RX_WAIT_TIMEOUT_MS;  // WS transactions could take longer
  } else {
    maxWaitTime = USB_RX_WAIT_TIMEOUT_MS; 
  }
  do{
    newReceivedData = false;  // this will be set when we get the response

    try{
      retryCount++;
      
      if(window.usingWS){   // Websocket (WiFI) or USB ?
        ws.send(toSendBuffer);
      } else {
        port.send(toSendBuffer);  // send the byte - the device will reply with OK/Error
      }

      if(expectResponse){
        timeout = 0;
        do{
          await sleep(1);       // Pointless to request the response immediately, device takes a couple of ms to process
          if(!window.usingWS){  // WS packets have a separate handler that sets newReceivedData to true
            await port.read();
          }
          timeout++;
        } while (newReceivedData==false && timeout < maxWaitTime);
      } else {
        break;  // No response expected so dont retry 
      }
    } catch(err) {
      console.log("Transaction attempt " + retryCount + " of " + maxPacketRetries + " failed: " + err);
    }

  } while(!newReceivedData && retryCount < maxPacketRetries);

  window.portBusy = false;

  if(expectResponse && !window.usingWS){
    // Check the response. CRC has already been validated, if it fails then newReceivedData will be False
    if(timeout >= maxWaitTime || !newReceivedData || retryCount > maxPacketRetries){
      console.log("Timed out waiting for device response");
      status("Device disconnected");
      disconnect_tasks();
      return -1;

    } else if(globalReceviedBuffer[0] === 0xFF){
      console.log("Device rejected the packet");
      status("Packet error, please unplug and try again");
      sendHumio('url,'    + window.location.href           + '\n' +
                'serial,' + window.SerialNumber.toString() + '\n' +
                'model,'  + window.ModelName.toString()    + '\n' +
                'message,'+ '"Error - packet ' + globalReceviedBuffer[1] + ' was rejected' + '"\n');
      return -1;
    }
  }

  return 1;
}

// Things to do when we're disconnected (USB or WS)
function disconnect_tasks(){

  window.connectWifiButton.disabled = false;
  window.connectButton.disabled = false;
  window.wifiWSIP.disabled = false;

  window.connectButton.textContent = 'Connect USB';
  window.connectWifiButton.textContent = 'Connect WiFi';
  if (location.protocol === 'https:') {
    // We are HTTPS for USB connection
    document.getElementById("connectWifi").textContent = "WiFi Connection..."; 

  } else if (location.protocol === 'http:') {
    // We are HTTP for websocket use
    document.getElementById("connect").textContent = "USB Connection..."; 
  }

  window.portBusy = false;
  newReceivedData = false;
  window.usingWS = false;
  port = null;
  
  // Update status chip
  document.getElementById("statusText").innerHTML = "Not Connected";
  document.getElementById("statusBubble").classList.remove("mdl-color--green","mdl-color--red");
  document.getElementById("statusBubble").classList.add("mdl-color--yellow");

  // disable controls
  document.getElementById("submitTime").disabled = true;
  document.getElementById("submitLED").disabled = true;
  document.getElementById("submitAnti").disabled = true;
  document.getElementById("submitOnOff").disabled = true;
  document.getElementById("submitWifi").disabled = true;
  document.getElementById("ALSthreshold").disabled = true; 
  document.getElementById("sliderRed").disabled = true;
  document.getElementById("sliderGreen").disabled = true;
  document.getElementById("sliderBlue").disabled = true;
  document.getElementById("enableVoiceDiv").style.display = "none";
  document.getElementById("VoiceAcctSetup").disabled = true;
  
  // Diagnostics updater and connectionChecker interval functions must be stopped
  window.clearInterval(diagUpdater);
  window.clearInterval(connectionChecker);
  diagUpdater = null;
  connectionChecker = null;
}

// Things to do on a successful connection (USB or WS)
function connect_tasks(){
  // Update status chip
  document.getElementById("statusText").innerHTML = "Connected";
  document.getElementById("statusBubble").classList.remove("mdl-color--yellow","mdl-color--red");
  document.getElementById("statusBubble").classList.add("mdl-color--green");

  // Enable controls
  document.getElementById("submitTime").disabled = false;
  document.getElementById("submitLED").disabled = false;
  document.getElementById("submitAnti").disabled = false;
  document.getElementById("submitOnOff").disabled = false;
  document.getElementById("submitWifi").disabled = false;
  document.getElementById("ALSthreshold").disabled = false; 
  document.getElementById("sliderRed").disabled = false;
  document.getElementById("sliderGreen").disabled = false;
  document.getElementById("sliderBlue").disabled = false;
  document.getElementById("VoiceAcctSetup").disabled = false;
}

// Full webapp only
window.enable_debug = function(){
  window.packet_data_debug = true;
  document.getElementById('cardWifi').style.display = "block";	// show 
  document.getElementById('cardVoiceAssistant').style.display = "block";
  connect_tasks();
}

  });
})();
